# Copyright (c) 2015 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from neutron_lib.api.definitions import external_net as extnet_apidef
from neutron_lib.api.definitions import l3 as l3_apidef
from neutron_lib.api.definitions import portbindings
from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib.exceptions import agent as agent_exc

from neutron_lib import constants
from neutron_lib import context

from neutron.api.rpc.handlers import l3_rpc
from neutron.common import constants as n_const
from neutron.common import topics
from neutron.tests.common import helpers
from neutron.tests.unit.plugins.ml2 import base as ml2_test_base


DEVICE_OWNER_COMPUTE = constants.DEVICE_OWNER_COMPUTE_PREFIX + 'fake'


class L3DvrTestCaseBase(ml2_test_base.ML2TestFramework):
    def setUp(self):
        super(L3DvrTestCaseBase, self).setUp()
        self.l3_agent = helpers.register_l3_agent(
            host="host0",
            agent_mode=constants.L3_AGENT_MODE_DVR_SNAT)
        # register OVS agents to avoid time wasted on committing
        # port binding failures on every port update
        helpers.register_ovs_agent(host='host1')
        helpers.register_ovs_agent(host='host2')

    def _create_router(self, distributed=True, ha=False, admin_state_up=True):
        return (super(L3DvrTestCaseBase, self).
                _create_router(distributed=distributed, ha=ha,
                               admin_state_up=admin_state_up))


class L3DvrTestCase(L3DvrTestCaseBase):
    def test_update_router_db_centralized_to_distributed(self):
        router = self._create_router(distributed=False)
        # router needs to be in admin state down in order to be upgraded to DVR
        self.l3_plugin.update_router(
            self.context, router['id'], {'router': {'admin_state_up': False}})
        self.assertFalse(router['distributed'])
        self.l3_plugin.update_router(
            self.context, router['id'], {'router': {'distributed': True}})
        router = self.l3_plugin.get_router(self.context, router['id'])
        self.assertTrue(router['distributed'])

    def test_get_device_owner_distributed_router_object(self):
        router = self._create_router()
        self.assertEqual(
            constants.DEVICE_OWNER_DVR_INTERFACE,
            self.l3_plugin._get_device_owner(self.context, router))

    def test_get_device_owner_distributed_router_id(self):
        router = self._create_router()
        self.assertEqual(
            constants.DEVICE_OWNER_DVR_INTERFACE,
            self.l3_plugin._get_device_owner(self.context, router['id']))

    def test_get_device_owner_centralized(self):
        router = self._create_router(distributed=False)
        self.assertEqual(
            constants.DEVICE_OWNER_ROUTER_INTF,
            self.l3_plugin._get_device_owner(self.context, router['id']))

    def test_get_agent_gw_ports_exist_for_network_no_port(self):
        self.assertIsNone(
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, 'network_id', 'host', 'agent_id'))

    def test_csnat_ports_are_created_and_deleted_based_on_router_subnet(self):
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        net1 = self._make_network(self.fmt, 'net1', True)
        subnet1 = self._make_subnet(
            self.fmt, net1, '10.1.0.1', '10.1.0.0/24', enable_dhcp=True)
        subnet2 = self._make_subnet(
            self.fmt, net1, '10.2.0.1', '10.2.0.0/24', enable_dhcp=True)
        ext_net = self._make_network(self.fmt, 'ext_net', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '20.0.0.1', '20.0.0.0/24', enable_dhcp=True)
        # Create first router and add an interface
        router1 = self._create_router()
        ext_net_id = ext_net['network']['id']
        net1_id = net1['network']['id']
        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router1['id'],
            {'network_id': ext_net_id})
        # Now add router interface (subnet1) from net1 to router
        self.l3_plugin.add_router_interface(
            self.context, router1['id'],
            {'subnet_id': subnet1['subnet']['id']})
        # Now add router interface (subnet2) from net1 to router
        self.l3_plugin.add_router_interface(
            self.context, router1['id'],
            {'subnet_id': subnet2['subnet']['id']})
        # Now check the valid snat interfaces passed to the agent
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        self.assertEqual(2, len(snat_router_intfs[router1['id']]))
        # Also make sure that there are no csnat ports created and
        # left over.
        csnat_ports = self.core_plugin.get_ports(
            self.context, filters={
                'network_id': [net1_id],
                'device_owner': [constants.DEVICE_OWNER_ROUTER_SNAT]})
        self.assertEqual(2, len(csnat_ports))
        # Now remove router interface (subnet1) from net1 to router
        self.l3_plugin.remove_router_interface(
            self.context, router1['id'],
            {'subnet_id': subnet1['subnet']['id']})
        # Now remove router interface (subnet2) from net1 to router
        self.l3_plugin.remove_router_interface(
            self.context, router1['id'],
            {'subnet_id': subnet2['subnet']['id']})
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        self.assertEqual(0, len(snat_router_intfs[router1['id']]))
        # Also make sure that there are no csnat ports created and
        # left over.
        csnat_ports = self.core_plugin.get_ports(
            self.context, filters={
                'network_id': [net1_id],
                'device_owner': [constants.DEVICE_OWNER_ROUTER_SNAT]})
        self.assertEqual(0, len(csnat_ports))

    def _test_remove_router_interface_leaves_snat_intact(self, by_subnet):
        with self.subnet() as subnet1, \
                self.subnet(cidr='20.0.0.0/24') as subnet2:
            kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                      extnet_apidef.EXTERNAL: True}
            with self.network(**kwargs) as ext_net, \
                    self.subnet(network=ext_net,
                                cidr='30.0.0.0/24'):
                router = self._create_router()
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet1['subnet']['id']})
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet2['subnet']['id']})
                gw_info = {'network_id': ext_net['network']['id']}
                self.l3_plugin.update_router(
                    self.context, router['id'],
                    {'router': {l3_apidef.EXTERNAL_GW_INFO: gw_info}})

                snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
                    self.context, [router['id']])
                self.assertEqual(
                    2, len(snat_router_intfs[router['id']]))

                if by_subnet:
                    self.l3_plugin.remove_router_interface(
                        self.context, router['id'],
                        {'subnet_id': subnet1['subnet']['id']})
                else:
                    port = self.core_plugin.get_ports(
                        self.context, filters={
                            'network_id': [subnet1['subnet']['network_id']],
                            'device_owner':
                                [constants.DEVICE_OWNER_DVR_INTERFACE]})[0]
                    self.l3_plugin.remove_router_interface(
                        self.context, router['id'],
                        {'port_id': port['id']})

                self.assertEqual(
                    1, len(self.l3_plugin._get_snat_sync_interfaces(
                        self.context, [router['id']])))

    def test_remove_router_interface_by_subnet_leaves_snat_intact(self):
        self._test_remove_router_interface_leaves_snat_intact(by_subnet=True)

    def test_remove_router_interface_by_port_leaves_snat_intact(self):
        self._test_remove_router_interface_leaves_snat_intact(
            by_subnet=False)

    def setup_create_agent_gw_port_for_network(self, network=None):
        if not network:
            network = self._make_network(self.fmt, '', True)
        network_id = network['network']['id']
        port = self.core_plugin.create_port(
            self.context,
            {'port': {'tenant_id': '',
                      'network_id': network_id,
                      'mac_address': constants.ATTR_NOT_SPECIFIED,
                      'fixed_ips': constants.ATTR_NOT_SPECIFIED,
                      'device_id': self.l3_agent['id'],
                      'device_owner': constants.DEVICE_OWNER_AGENT_GW,
                      portbindings.HOST_ID: '',
                      'admin_state_up': True,
                      'name': ''}})
        return network_id, port

    def test_get_agent_gw_port_for_network(self):
        network_id, port = (
            self.setup_create_agent_gw_port_for_network())

        self.assertEqual(
            port['id'],
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, network_id, None, self.l3_agent['id'])['id'])

    def test_delete_agent_gw_port_for_network(self):
        network_id, port = (
            self.setup_create_agent_gw_port_for_network())

        self.l3_plugin.delete_floatingip_agent_gateway_port(
            self.context, "", network_id)
        self.assertIsNone(
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, network_id, "", self.l3_agent['id']))

    def test_get_fip_agent_gw_ports(self):
        self.setup_create_agent_gw_port_for_network()

        self.assertEqual(
            1, len(self.l3_plugin._get_fip_agent_gw_ports(
                self.context, self.l3_agent['id'])))

    def test_process_routers(self):
        router = self._create_router()
        if not router.get('gw_port_id'):
            router['gw_port_id'] = 'fake_gw_id'
        self.l3_plugin._get_fip_agent_gw_ports = mock.Mock(
            return_value='fip_interface')
        self.l3_plugin._get_snat_sync_interfaces = mock.Mock(
            return_value={router['id']: 'snat_interface'})
        result = self.l3_plugin._process_routers(self.context, [router],
                                                 self.l3_agent)

        self.assertEqual(
            router['id'], result[router['id']]['id'])
        self.assertIn(n_const.FLOATINGIP_AGENT_INTF_KEY, result[router['id']])
        self.l3_plugin._get_fip_agent_gw_ports.assert_called_once_with(
            self.context, self.l3_agent['id'])
        self.l3_plugin._get_snat_sync_interfaces.assert_called_once_with(
            self.context, [router['id']])

    def test_agent_gw_port_delete_when_last_gateway_for_ext_net_removed(self):
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        net1 = self._make_network(self.fmt, 'net1', True)
        net2 = self._make_network(self.fmt, 'net2', True)
        subnet1 = self._make_subnet(
            self.fmt, net1, '10.1.0.1', '10.1.0.0/24', enable_dhcp=True)
        subnet2 = self._make_subnet(
            self.fmt, net2, '10.1.0.1', '10.1.0.0/24', enable_dhcp=True)
        ext_net = self._make_network(self.fmt, 'ext_net', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '20.0.0.1', '20.0.0.0/24', enable_dhcp=True)
        # Create first router and add an interface
        router1 = self._create_router()
        ext_net_id = ext_net['network']['id']
        self.l3_plugin.add_router_interface(
            self.context, router1['id'],
            {'subnet_id': subnet1['subnet']['id']})
        # Set gateway to first router
        self.l3_plugin._update_router_gw_info(
            self.context, router1['id'],
            {'network_id': ext_net_id})
        # Create second router and add an interface
        router2 = self._create_router()
        self.l3_plugin.add_router_interface(
            self.context, router2['id'],
            {'subnet_id': subnet2['subnet']['id']})
        # Set gateway to second router
        self.l3_plugin._update_router_gw_info(
            self.context, router2['id'],
            {'network_id': ext_net_id})
        # Create an agent gateway port for the external network
        net_id, agent_gw_port = (
            self.setup_create_agent_gw_port_for_network(network=ext_net))
        # Check for agent gateway ports
        self.assertIsNotNone(
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, ext_net_id, "", self.l3_agent['id']))
        self.l3_plugin._update_router_gw_info(
            self.context, router1['id'], {})
        # Check for agent gateway port after deleting one of the gw
        self.assertIsNotNone(
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, ext_net_id, "", self.l3_agent['id']))
        self.l3_plugin._update_router_gw_info(
            self.context, router2['id'], {})
        # Check for agent gateway port after deleting last gw
        self.assertIsNone(
            self.l3_plugin._get_agent_gw_ports_exist_for_network(
                self.context, ext_net_id, "", self.l3_agent['id']))

    def test_create_floating_ip_with_no_dvr_agents(self):
        self._test_create_floating_ip_agent_notification(
            test_agent_mode=None)

    def _test_create_floating_ip_agent_notification(
        self, dvr=True, test_agent_mode=constants.L3_AGENT_MODE_DVR):
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as int_subnet,\
                self.port(subnet=int_subnet,
                          device_owner=DEVICE_OWNER_COMPUTE) as int_port:
            self.core_plugin.update_port(
                self.context, int_port['port']['id'],
                {'port': {portbindings.HOST_ID: 'host1'}})
            # and create l3 agents on corresponding hosts
            if test_agent_mode is not None:
                helpers.register_l3_agent(host='host1',
                    agent_mode=test_agent_mode)

            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})

            router = self._create_router(distributed=dvr)
            self.l3_plugin.update_router(
                self.context, router['id'],
                {'router': {
                    'external_gateway_info': {'network_id': ext_net_id}}})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': int_subnet['subnet']['id']})
            if test_agent_mode is None:
                self.assertRaises(
                    agent_exc.AgentNotFoundByTypeHost,
                    self.l3_plugin.create_fip_agent_gw_port_if_not_exists,
                    self.context,
                    ext_net_id,
                    'host1')
                return
            floating_ip = {'floating_network_id': ext_net_id,
                           'router_id': router['id'],
                           'port_id': int_port['port']['id'],
                           'tenant_id': int_port['port']['tenant_id'],
                           'dns_name': '', 'dns_domain': ''}
            with mock.patch.object(
                    self.l3_plugin, '_l3_rpc_notifier') as l3_notif:
                self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})
                if dvr:
                    if test_agent_mode == (
                        n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL):
                        if router['ha']:
                            expected_calls = [
                                mock.call(self.context,
                                          [router['id']], 'host0'),
                                mock.call(self.context,
                                          [router['id']], 'standby')]
                            l3_notif.routers_updated_on_host.assert_has_calls(
                                expected_calls, any_order=True)
                            self.assertFalse(l3_notif.routers_updated.called)
                        if not router['ha']:
                            l3_notif.routers_updated_on_host.\
                                assert_called_once_with(
                                    self.context, [router['id']], 'host0')
                            self.assertFalse(l3_notif.routers_updated.called)
                    else:
                        l3_notif.routers_updated_on_host.\
                            assert_called_once_with(
                                self.context, [router['id']], 'host1')
                        self.assertFalse(l3_notif.routers_updated.called)
                else:
                    l3_notif.routers_updated.assert_called_once_with(
                        self.context, [router['id']], None)
                    self.assertFalse(
                        l3_notif.routers_updated_on_host.called)

    def test_create_floating_ip_agent_notification(self):
        self._test_create_floating_ip_agent_notification()

    def test_create_floating_ip_agent_notification_for_dvr_no_external_agent(
        self):
        agent_mode = n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL
        self._test_create_floating_ip_agent_notification(
            test_agent_mode=agent_mode)

    def test_create_floating_ip_agent_notification_non_dvr(self):
        self._test_create_floating_ip_agent_notification(dvr=False)

    def _test_update_floating_ip_agent_notification(
        self, dvr=True, test_agent_mode=constants.L3_AGENT_MODE_DVR):
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as int_subnet1,\
                self.subnet(cidr='30.0.0.0/24') as int_subnet2,\
                self.port(subnet=int_subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE) as int_port1,\
                self.port(subnet=int_subnet2,
                          device_owner=DEVICE_OWNER_COMPUTE) as int_port2:
            # locate internal ports on different hosts
            self.core_plugin.update_port(
                self.context, int_port1['port']['id'],
                {'port': {portbindings.HOST_ID: 'host1'}})
            self.core_plugin.update_port(
                self.context, int_port2['port']['id'],
                {'port': {portbindings.HOST_ID: 'host2'}})
            # and create l3 agents on corresponding hosts
            helpers.register_l3_agent(host='host1',
                agent_mode=test_agent_mode)
            helpers.register_l3_agent(host='host2',
                agent_mode=test_agent_mode)

            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})

            router1 = self._create_router(distributed=dvr)
            router2 = self._create_router(distributed=dvr)
            for router in (router1, router2):
                self.l3_plugin.update_router(
                    self.context, router['id'],
                    {'router': {
                        'external_gateway_info': {'network_id': ext_net_id}}})
            self.l3_plugin.add_router_interface(
                self.context, router1['id'],
                {'subnet_id': int_subnet1['subnet']['id']})
            self.l3_plugin.add_router_interface(
                self.context, router2['id'],
                {'subnet_id': int_subnet2['subnet']['id']})

            floating_ip = {'floating_network_id': ext_net_id,
                           'router_id': router1['id'],
                           'port_id': int_port1['port']['id'],
                           'tenant_id': int_port1['port']['tenant_id'],
                           'dns_name': '', 'dns_domain': ''}
            floating_ip = self.l3_plugin.create_floatingip(
                self.context, {'floatingip': floating_ip})

            with mock.patch.object(
                    self.l3_plugin, '_l3_rpc_notifier') as l3_notif:
                updated_floating_ip = {'router_id': router2['id'],
                                       'port_id': int_port2['port']['id']}
                self.l3_plugin.update_floatingip(
                    self.context, floating_ip['id'],
                    {'floatingip': updated_floating_ip})
                if dvr:
                    if test_agent_mode == (
                        n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL):
                        if router1['ha'] and router2['ha']:
                            self.assertEqual(
                                4,
                                l3_notif.routers_updated_on_host.call_count)
                            expected_calls = [
                                mock.call(self.context,
                                          [router1['id']], 'host0'),
                                mock.call(self.context,
                                          [router1['id']], 'standby'),
                                mock.call(self.context,
                                          [router2['id']], 'host0'),
                                mock.call(self.context,
                                          [router2['id']], 'standby')]
                            l3_notif.routers_updated_on_host.assert_has_calls(
                                expected_calls, any_order=True)
                            self.assertFalse(l3_notif.routers_updated.called)
                        else:
                            self.assertEqual(
                                2,
                                l3_notif.routers_updated_on_host.call_count)
                            expected_calls = [
                                mock.call(self.context,
                                          [router1['id']], 'host0'),
                                mock.call(self.context,
                                          [router2['id']], 'host0')]
                            l3_notif.routers_updated_on_host.assert_has_calls(
                                expected_calls)
                            self.assertFalse(l3_notif.routers_updated.called)
                    else:
                        self.assertEqual(
                            2, l3_notif.routers_updated_on_host.call_count)
                        expected_calls = [
                            mock.call(self.context, [router1['id']], 'host1'),
                            mock.call(self.context, [router2['id']], 'host2')]
                        l3_notif.routers_updated_on_host.assert_has_calls(
                            expected_calls)
                        self.assertFalse(l3_notif.routers_updated.called)
                else:
                    self.assertEqual(
                        2, l3_notif.routers_updated.call_count)
                    expected_calls = [
                        mock.call(self.context, [router1['id']], None),
                        mock.call(self.context, [router2['id']], None)]
                    l3_notif.routers_updated.assert_has_calls(
                        expected_calls)
                    self.assertFalse(l3_notif.routers_updated_on_host.called)

    def test_update_floating_ip_agent_notification(self):
        self._test_update_floating_ip_agent_notification()

    def test_update_floating_ip_agent_notification_with_dvr_no_external_agents(
        self):
        agent_mode = n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL
        self._test_update_floating_ip_agent_notification(
            test_agent_mode=agent_mode)

    def test_update_floating_ip_agent_notification_non_dvr(self):
        self._test_update_floating_ip_agent_notification(dvr=False)

    def test_delete_floating_ip_with_no_agents(self):
        self._test_delete_floating_ip_agent_notification(
            test_agent_mode=None)

    def _test_delete_floating_ip_agent_notification(
        self, dvr=True, test_agent_mode=constants.L3_AGENT_MODE_DVR):
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as int_subnet,\
                self.port(subnet=int_subnet,
                          device_owner=DEVICE_OWNER_COMPUTE) as int_port:
            self.core_plugin.update_port(
                self.context, int_port['port']['id'],
                {'port': {portbindings.HOST_ID: 'host1'}})
            # and create l3 agents on corresponding hosts
            helpers.register_l3_agent(host='host1',
                agent_mode=test_agent_mode)
            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})

            router = self._create_router(distributed=dvr)
            self.l3_plugin.update_router(
                self.context, router['id'],
                {'router': {
                    'external_gateway_info': {'network_id': ext_net_id}}})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': int_subnet['subnet']['id']})

            floating_ip = {'floating_network_id': ext_net_id,
                           'router_id': router['id'],
                           'port_id': int_port['port']['id'],
                           'tenant_id': int_port['port']['tenant_id'],
                           'dns_name': '', 'dns_domain': ''}
            floating_ip = self.l3_plugin.create_floatingip(
                self.context, {'floatingip': floating_ip})
            if test_agent_mode is None:
                with mock.patch.object(
                        self.l3_plugin, 'get_dvr_agent_on_host') as a_mock,\
                        mock.patch.object(self.l3_plugin,
                                          '_l3_rpc_notifier') as l3_notif:
                    a_mock.return_value = None
                    self.l3_plugin.delete_floatingip(
                        self.context, floating_ip['id'])
                    self.assertFalse(
                        l3_notif.routers_updated_on_host.called)
                    return
            with mock.patch.object(
                        self.l3_plugin, '_l3_rpc_notifier') as l3_notif:
                self.l3_plugin.delete_floatingip(
                    self.context, floating_ip['id'])
                if dvr:
                    if test_agent_mode == (
                        n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL):
                        if router['ha']:
                            expected_calls = [
                                mock.call(self.context,
                                          [router['id']], 'host0'),
                                mock.call(self.context,
                                          [router['id']], 'standby')]
                            l3_notif.routers_updated_on_host.assert_has_calls(
                                expected_calls, any_order=True)
                            self.assertFalse(l3_notif.routers_updated.called)
                        else:
                            l3_notif.routers_updated_on_host.\
                                assert_called_once_with(
                                    self.context, [router['id']], 'host0')
                            self.assertFalse(l3_notif.routers_updated.called)
                    if test_agent_mode == (
                            constants.L3_AGENT_MODE_DVR):
                        l3_notif.routers_updated_on_host.\
                            assert_called_once_with(
                                self.context, [router['id']], 'host1')
                        self.assertFalse(l3_notif.routers_updated.called)
                else:
                    l3_notif.routers_updated.assert_called_once_with(
                        self.context, [router['id']], None)
                    self.assertFalse(
                        l3_notif.routers_updated_on_host.called)

    def test_delete_floating_ip_agent_notification(self):
        self._test_delete_floating_ip_agent_notification()

    def test_delete_floating_ip_agent_notification_with_dvr_no_external_agents(
        self):
        agent_mode = n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL
        self._test_delete_floating_ip_agent_notification(
            test_agent_mode=agent_mode)

    def test_delete_floating_ip_agent_notification_non_dvr(self):
        self._test_delete_floating_ip_agent_notification(dvr=False)

    def test_router_with_ipv4_and_multiple_ipv6_on_same_network(self):
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.0.0.1', '10.0.0.0/24',
            ip_version=4, enable_dhcp=True)
        self._make_subnet(
            self.fmt, ext_net, '2001:db8::1', '2001:db8::/64',
            ip_version=6, enable_dhcp=True)
        router1 = self._create_router()
        self.l3_plugin._update_router_gw_info(
            self.context, router1['id'],
            {'network_id': ext_net['network']['id']})
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        self.assertEqual(0, len(snat_router_intfs[router1['id']]))
        private_net1 = self._make_network(self.fmt, 'net1', True)
        private_ipv6_subnet1 = self._make_subnet(self.fmt,
            private_net1, 'fd00::1',
            cidr='fd00::1/64', ip_version=6,
            ipv6_ra_mode='slaac',
            ipv6_address_mode='slaac')
        private_ipv6_subnet2 = self._make_subnet(self.fmt,
            private_net1, 'fd01::1',
            cidr='fd01::1/64', ip_version=6,
            ipv6_ra_mode='slaac',
            ipv6_address_mode='slaac')
        # Add the first IPv6 subnet to the router
        self.l3_plugin.add_router_interface(
            self.context, router1['id'],
            {'subnet_id': private_ipv6_subnet1['subnet']['id']})
        # Check for the internal snat port interfaces
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        self.assertEqual(1, len(snat_router_intfs[router1['id']]))
        # Add the second IPv6 subnet to the router
        self.l3_plugin.add_router_interface(
            self.context, router1['id'],
            {'subnet_id': private_ipv6_subnet2['subnet']['id']})
        # Check for the internal snat port interfaces
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        snat_intf_list = snat_router_intfs[router1['id']]
        fixed_ips = snat_intf_list[0]['fixed_ips']
        self.assertEqual(1, len(snat_router_intfs[router1['id']]))
        self.assertEqual(2, len(fixed_ips))
        # Now delete the router interface and it should update the
        # SNAT port with the right fixed_ips instead of deleting it.
        self.l3_plugin.remove_router_interface(
            self.context, router1['id'],
            {'subnet_id': private_ipv6_subnet2['subnet']['id']})
        # Check for the internal snat port interfaces
        snat_router_intfs = self.l3_plugin._get_snat_sync_interfaces(
            self.context, [router1['id']])
        snat_intf_list = snat_router_intfs[router1['id']]
        fixed_ips = snat_intf_list[0]['fixed_ips']
        self.assertEqual(1, len(snat_router_intfs[router1['id']]))
        self.assertEqual(1, len(fixed_ips))

    def test_unbound_allowed_addr_pairs_fip_with_multiple_active_vms(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        HOST2 = 'host2'
        helpers.register_l3_agent(
            host=HOST2, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        test_allocation_pools = [{'start': '10.1.0.2',
                                  'end': '10.1.0.20'}]
        fixed_vrrp_ip = [{'ip_address': '10.1.0.201'}]
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])

        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            allocation_pools=test_allocation_pools,
            enable_dhcp=True)
        vrrp_port = self._make_port(
            self.fmt,
            private_net1['network']['id'],
            device_owner='',
            fixed_ips=fixed_vrrp_ip)
        allowed_address_pairs = [
            {'ip_address': '10.1.0.201',
             'mac_address': vrrp_port['port']['mac_address']}]
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port1,\
                self.port(
                    subnet=private_subnet1,
                    device_owner=DEVICE_OWNER_COMPUTE) as int_port2:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                vm_port1 = self.core_plugin.update_port(
                    self.context, int_port1['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                vm_port2 = self.core_plugin.update_port(
                    self.context, int_port2['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST2}})
                vrrp_port_db = self.core_plugin.get_port(
                    self.context, vrrp_port['port']['id'])
                # Make sure that the VRRP port is not bound to any host
                self.assertNotEqual(vrrp_port_db[portbindings.HOST_ID], HOST1)
                self.assertNotEqual(vrrp_port_db[portbindings.HOST_ID], HOST2)
                self.assertNotEqual(
                    vrrp_port_db[portbindings.HOST_ID], self.l3_agent['host'])
                # Now update both the VM ports with the allowed_address_pair ip
                self.core_plugin.update_port(
                     self.context, vm_port1['id'],
                     {'port': {
                         'allowed_address_pairs': allowed_address_pairs}})
                updated_vm_port1 = self.core_plugin.get_port(
                    self.context, vm_port1['id'])
                expected_allowed_address_pairs1 = updated_vm_port1.get(
                    'allowed_address_pairs')
                self.assertEqual(expected_allowed_address_pairs1,
                                 allowed_address_pairs)
                self.core_plugin.update_port(
                    self.context, vm_port2['id'],
                    {'port': {
                        'allowed_address_pairs': allowed_address_pairs}})
                updated_vm_port2 = self.core_plugin.get_port(
                    self.context, vm_port2['id'])
                expected_allowed_address_pairs2 = updated_vm_port2.get(
                    'allowed_address_pairs')
                self.assertEqual(expected_allowed_address_pairs2,
                                 allowed_address_pairs)
                # Now let us assign the floatingip to the vrrp port that is
                # unbound to any host.
                floating_ip = {'floating_network_id': ext_net['network']['id'],
                               'router_id': router['id'],
                               'port_id': vrrp_port['port']['id'],
                               'tenant_id': vrrp_port['port']['tenant_id']}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})
                expected_routers_updated_calls = [
                    mock.call(self.context, mock.ANY, 'host0'),
                    mock.call(self.context, mock.ANY, HOST1),
                    mock.call(self.context, mock.ANY, HOST1),
                    mock.call(self.context, mock.ANY, HOST2)]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls, any_order=True)
                self.assertFalse(l3_notifier.routers_updated.called)
                router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, self.l3_agent['host'], [router['id']]))
                floatingips = router_info[0][constants.FLOATINGIP_KEY]
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])

    def test_dvr_process_floatingips_for_dvr_on_full_sync(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        # Schedule the router to the dvr_snat node
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])

        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            enable_dhcp=True)
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port1,\
                self.port(
                    subnet=private_subnet1) as int_port2:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                self.core_plugin.update_port(
                    self.context, int_port1['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                # Now let us assign the floatingip to the bound port
                # and unbound port.
                fip1 = {'floating_network_id': ext_net['network']['id'],
                        'router_id': router['id'],
                        'port_id': int_port1['port']['id'],
                        'tenant_id': int_port1['port']['tenant_id']}
                self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': fip1})
                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, HOST1)]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls)
                self.assertFalse(l3_notifier.routers_updated.called)
                fip2 = {'floating_network_id': ext_net['network']['id'],
                        'router_id': router['id'],
                        'port_id': int_port2['port']['id'],
                        'tenant_id': int_port2['port']['tenant_id']}
                self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': fip2})
                router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, self.l3_agent['host'], [router['id']]))
                floatingips = router_info[0][constants.FLOATINGIP_KEY]
                self.assertEqual(1, len(floatingips))
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])
                self.assertEqual(n_const.FLOATING_IP_HOST_NEEDS_BINDING,
                                 floatingips[0]['host'])
                router1_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, HOST1, [router['id']]))
                floatingips = router1_info[0][constants.FLOATINGIP_KEY]
                self.assertEqual(1, len(floatingips))
                self.assertEqual(HOST1, floatingips[0]['host'])
                self.assertIsNone(floatingips[0]['dest_host'])

    def test_dvr_router_unbound_floating_ip_migrate_to_bound_host(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])

        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            enable_dhcp=True)
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port1:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                # Next we can try to associate the floatingip to the
                # VM port
                floating_ip = {'floating_network_id': ext_net['network']['id'],
                               'router_id': router['id'],
                               'port_id': int_port1['port']['id'],
                               'tenant_id': int_port1['port']['tenant_id']}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})

                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, 'host0')]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls)
                self.assertFalse(l3_notifier.routers_updated.called)
                router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, self.l3_agent['host'], [router['id']]))
                floatingips = router_info[0][constants.FLOATINGIP_KEY]
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])
                # Now do the host binding to the fip port
                self.core_plugin.update_port(
                    self.context, int_port1['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, 'host0'),
                        mock.call(self.context, mock.ANY, HOST1)]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls)
                updated_router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, HOST1, [router['id']]))
                floatingips = updated_router_info[0][constants.FLOATINGIP_KEY]
                self.assertFalse(floatingips[0].get(n_const.DVR_SNAT_BOUND))
                self.assertEqual(HOST1, floatingips[0]['host'])

    def test_dvr_router_centralized_floating_ip(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=n_const.L3_AGENT_MODE_DVR_NO_EXTERNAL)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])

        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            enable_dhcp=True)
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port1:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                vm_port = self.core_plugin.update_port(
                    self.context, int_port1['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                self.assertEqual(
                    1, l3_notifier.routers_updated_on_host.call_count)
                # Next we can try to associate the floatingip to the
                # VM port
                floating_ip = {'floating_network_id': ext_net['network']['id'],
                               'router_id': router['id'],
                               'port_id': vm_port['id'],
                               'tenant_id': vm_port['tenant_id']}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})

                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, HOST1),
                        mock.call(self.context, mock.ANY, 'host0')]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls)
                self.assertFalse(l3_notifier.routers_updated.called)
                router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, self.l3_agent['host'], [router['id']]))
                floatingips = router_info[0][constants.FLOATINGIP_KEY]
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])
                # Test case to make sure when an agent in this case
                # dvr_no_external restarts and does a full sync, we need
                # to make sure that the returned router_info has
                # DVR_SNAT_BOUND flag enabled, otherwise the floating IP
                # state would error out.
                router_sync_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, HOST1, [router['id']]))
                floatingips = router_sync_info[0][constants.FLOATINGIP_KEY]
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])

    def test_allowed_addr_pairs_delayed_fip_and_update_arp_entry(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        HOST2 = 'host2'
        helpers.register_l3_agent(
            host=HOST2, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        test_allocation_pools = [{'start': '10.1.0.2',
                                  'end': '10.1.0.20'}]
        fixed_vrrp_ip = [{'ip_address': '10.1.0.201'}]
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])

        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            allocation_pools=test_allocation_pools,
            enable_dhcp=True)
        vrrp_port = self._make_port(
            self.fmt,
            private_net1['network']['id'],
            fixed_ips=fixed_vrrp_ip)
        allowed_address_pairs = [
            {'ip_address': '10.1.0.201',
             'mac_address': vrrp_port['port']['mac_address']}]
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port,\
                self.port(subnet=private_subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE) as int_port2:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                vm_port = self.core_plugin.update_port(
                    self.context, int_port['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                vm_port_mac = vm_port['mac_address']
                vm_port_fixed_ips = vm_port['fixed_ips']
                vm_port_subnet_id = vm_port_fixed_ips[0]['subnet_id']
                vm_arp_table = {
                    'ip_address': vm_port_fixed_ips[0]['ip_address'],
                    'mac_address': vm_port_mac,
                    'subnet_id': vm_port_subnet_id}
                vm_port2 = self.core_plugin.update_port(
                    self.context, int_port2['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST2}})
                # Now update the VM port with the allowed_address_pair
                self.core_plugin.update_port(
                     self.context, vm_port['id'],
                     {'port': {
                         'allowed_address_pairs': allowed_address_pairs}})
                self.core_plugin.update_port(
                     self.context, vm_port2['id'],
                     {'port': {
                         'allowed_address_pairs': allowed_address_pairs}})
                self.assertEqual(
                    3, l3_notifier.routers_updated_on_host.call_count)
                updated_vm_port1 = self.core_plugin.get_port(
                    self.context, vm_port['id'])
                updated_vm_port2 = self.core_plugin.get_port(
                    self.context, vm_port2['id'])
                expected_allowed_address_pairs = updated_vm_port1.get(
                    'allowed_address_pairs')
                self.assertEqual(expected_allowed_address_pairs,
                                 allowed_address_pairs)
                expected_allowed_address_pairs_2 = updated_vm_port2.get(
                    'allowed_address_pairs')
                self.assertEqual(expected_allowed_address_pairs_2,
                                 allowed_address_pairs)
                # Now the VRRP port is attached to the VM port. At this
                # point, the VRRP port should not have inherited the
                # port host bindings from the parent VM port.
                cur_vrrp_port_db = self.core_plugin.get_port(
                    self.context, vrrp_port['port']['id'])
                self.assertNotEqual(
                    cur_vrrp_port_db[portbindings.HOST_ID], HOST1)
                self.assertNotEqual(
                    cur_vrrp_port_db[portbindings.HOST_ID], HOST2)
                # Next we can try to associate the floatingip to the
                # VRRP port that is already attached to the VM port
                floating_ip = {'floating_network_id': ext_net['network']['id'],
                               'router_id': router['id'],
                               'port_id': vrrp_port['port']['id'],
                               'tenant_id': vrrp_port['port']['tenant_id']}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})

                post_update_vrrp_port_db = self.core_plugin.get_port(
                    self.context, vrrp_port['port']['id'])
                vrrp_port_fixed_ips = post_update_vrrp_port_db['fixed_ips']
                vrrp_port_subnet_id = vrrp_port_fixed_ips[0]['subnet_id']
                vrrp_arp_table1 = {
                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
                    'mac_address': vm_port_mac,
                    'subnet_id': vrrp_port_subnet_id}

                expected_calls = [
                        mock.call(self.context,
                                  router['id'], vm_arp_table),
                        mock.call(self.context,
                                  router['id'], vrrp_arp_table1)]
                l3_notifier.add_arp_entry.assert_has_calls(
                        expected_calls)
                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, HOST1),
                        mock.call(self.context, mock.ANY, HOST2),
                        mock.call(self.context, mock.ANY, HOST1),
                        mock.call(self.context, mock.ANY, 'host0')]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls, any_order=True)
                self.assertFalse(l3_notifier.routers_updated.called)
                router_info = (
                    self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                        self.context, self.l3_agent['host'], [router['id']]))
                floatingips = router_info[0][constants.FLOATINGIP_KEY]
                self.assertTrue(floatingips[0][n_const.DVR_SNAT_BOUND])

    def test_dvr_gateway_host_binding_is_set(self):
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])
        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            enable_dhcp=True)
        self.l3_plugin.add_router_interface(
            self.context, router['id'],
            {'subnet_id': private_subnet1['subnet']['id']})
        # Check for the gw_port_host in the router dict to make
        # sure that the _build_routers_list in l3_dvr_db is called.
        router_handle = (
            self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                self.context, self.l3_agent['host'], [router['id']]))
        self.assertEqual(self.l3_agent['host'],
                         router_handle[0]['gw_port_host'])

    def test_allowed_address_pairs_update_arp_entry(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router(ha=False)
        private_net1 = self._make_network(self.fmt, 'net1', True)
        test_allocation_pools = [{'start': '10.1.0.2',
                                  'end': '10.1.0.20'}]
        fixed_vrrp_ip = [{'ip_address': '10.1.0.201'}]
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        ext_net = self._make_network(self.fmt, '', True, **kwargs)
        self._make_subnet(
            self.fmt, ext_net, '10.20.0.1', '10.20.0.0/24',
            ip_version=4, enable_dhcp=True)
        self.l3_plugin.schedule_router(self.context,
                                       router['id'],
                                       candidates=[self.l3_agent])
        # Set gateway to router
        self.l3_plugin._update_router_gw_info(
            self.context, router['id'],
            {'network_id': ext_net['network']['id']})
        private_subnet1 = self._make_subnet(
            self.fmt,
            private_net1,
            '10.1.0.1',
            cidr='10.1.0.0/24',
            ip_version=4,
            allocation_pools=test_allocation_pools,
            enable_dhcp=True)
        vrrp_port = self._make_port(
            self.fmt,
            private_net1['network']['id'],
            fixed_ips=fixed_vrrp_ip)
        allowed_address_pairs = [
            {'ip_address': '10.1.0.201',
             'mac_address': vrrp_port['port']['mac_address']}]
        with self.port(
                subnet=private_subnet1,
                device_owner=DEVICE_OWNER_COMPUTE) as int_port:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': private_subnet1['subnet']['id']})
            router_handle = (
                self.l3_plugin.list_active_sync_routers_on_active_l3_agent(
                    self.context, self.l3_agent['host'], [router['id']]))
            self.assertEqual(self.l3_agent['host'],
                             router_handle[0]['gw_port_host'])
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                vm_port = self.core_plugin.update_port(
                    self.context, int_port['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})
                vm_port_mac = vm_port['mac_address']
                vm_port_fixed_ips = vm_port['fixed_ips']
                vm_port_subnet_id = vm_port_fixed_ips[0]['subnet_id']
                vm_arp_table = {
                    'ip_address': vm_port_fixed_ips[0]['ip_address'],
                    'mac_address': vm_port_mac,
                    'subnet_id': vm_port_subnet_id}
                self.assertEqual(1, l3_notifier.add_arp_entry.call_count)
                floating_ip = {'floating_network_id': ext_net['network']['id'],
                               'router_id': router['id'],
                               'port_id': vrrp_port['port']['id'],
                               'tenant_id': vrrp_port['port']['tenant_id']}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})
                vrrp_port_db = self.core_plugin.get_port(
                    self.context, vrrp_port['port']['id'])
                self.assertNotEqual(vrrp_port_db[portbindings.HOST_ID], HOST1)
                # Now update the VM port with the allowed_address_pair
                self.core_plugin.update_port(
                     self.context, vm_port['id'],
                     {'port': {
                         'allowed_address_pairs': allowed_address_pairs}})
                updated_vm_port = self.core_plugin.get_port(
                    self.context, vm_port['id'])
                expected_allowed_address_pairs = updated_vm_port.get(
                    'allowed_address_pairs')
                self.assertEqual(expected_allowed_address_pairs,
                                 allowed_address_pairs)
                cur_vrrp_port_db = self.core_plugin.get_port(
                    self.context, vrrp_port['port']['id'])
                vrrp_port_fixed_ips = cur_vrrp_port_db['fixed_ips']
                vrrp_port_subnet_id = vrrp_port_fixed_ips[0]['subnet_id']
                vrrp_arp_table1 = {
                    'ip_address': vrrp_port_fixed_ips[0]['ip_address'],
                    'mac_address': vm_port_mac,
                    'subnet_id': vrrp_port_subnet_id}

                expected_calls = [
                        mock.call(self.context,
                                  router['id'], vm_arp_table),
                        mock.call(self.context,
                                  router['id'], vrrp_arp_table1)]
                l3_notifier.add_arp_entry.assert_has_calls(
                        expected_calls)
                expected_routers_updated_calls = [
                        mock.call(self.context, mock.ANY, HOST1),
                        mock.call(self.context, mock.ANY, 'host0')]
                l3_notifier.routers_updated_on_host.assert_has_calls(
                        expected_routers_updated_calls)
                self.assertFalse(l3_notifier.routers_updated.called)

    def test_update_vm_port_host_router_update(self):
        # register l3 agents in dvr mode in addition to existing dvr_snat agent
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        HOST2 = 'host2'
        helpers.register_l3_agent(
            host=HOST2, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router()
        with self.subnet() as subnet:
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})

            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier,\
                    self.port(subnet=subnet,
                              device_owner=DEVICE_OWNER_COMPUTE) as port:
                self.l3_plugin.agent_notifiers[
                    constants.AGENT_TYPE_L3] = l3_notifier
                self.core_plugin.update_port(
                    self.context, port['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST1}})

                l3_notifier.routers_updated_on_host.assert_called_once_with(
                    self.context, {router['id']}, HOST1)
                self.assertFalse(l3_notifier.routers_updated.called)

                # updating port's host (instance migration)
                l3_notifier.reset_mock()
                self.core_plugin.update_port(
                    self.context, port['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST2}})

                l3_notifier.routers_updated_on_host.assert_called_once_with(
                    self.context, {router['id']}, HOST2)
                l3_notifier.router_removed_from_agent.assert_called_once_with(
                    mock.ANY, router['id'], HOST1)

    def test_dvr_router_manual_rescheduling_removes_router(self):
        router = self._create_router()
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        with self.network(**kwargs) as ext_net,\
                self.subnet(network=ext_net),\
                self.subnet(cidr='20.0.0.0/24') as subnet,\
                self.port(subnet=subnet):
            self.l3_plugin._update_router_gw_info(
                self.context, router['id'],
                {'network_id': ext_net['network']['id']})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})
            self.l3_plugin.schedule_router(self.context,
                                           router['id'],
                                           candidates=[self.l3_agent])
            # Now the VM should be also scheduled on the node
            notifier = self.l3_plugin.agent_notifiers[
                constants.AGENT_TYPE_L3]
            with mock.patch.object(
                    notifier, 'router_removed_from_agent') as rtr_remove_mock:
                self.l3_plugin.remove_router_from_l3_agent(
                    self.context, self.l3_agent['id'], router['id'])
                rtr_remove_mock.assert_called_once_with(
                    self.context, router['id'], self.l3_agent['host'])

    def test_dvr_router_manual_rescheduling_updates_router(self):
        router = self._create_router()
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        with self.network(**kwargs) as ext_net,\
                self.subnet(network=ext_net),\
                self.subnet(cidr='20.0.0.0/24') as subnet,\
                self.port(subnet=subnet,
                          device_owner=DEVICE_OWNER_COMPUTE) as port:
            self.core_plugin.update_port(
                self.context, port['port']['id'],
                {'port': {'binding:host_id': self.l3_agent['host']}})
            self.l3_plugin._update_router_gw_info(
                self.context, router['id'],
                {'network_id': ext_net['network']['id']})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})
            self.l3_plugin.schedule_router(self.context,
                                           router['id'],
                                           candidates=[self.l3_agent])
            # Now the VM should be also scheduled on the node
            notifier = self.l3_plugin.agent_notifiers[
                constants.AGENT_TYPE_L3]
            with mock.patch.object(
                    notifier, 'routers_updated_on_host') as rtr_update_mock:
                self.l3_plugin.remove_router_from_l3_agent(
                    self.context, self.l3_agent['id'], router['id'])
                rtr_update_mock.assert_called_once_with(
                    self.context, [router['id']], self.l3_agent['host'])

    def _test_router_remove_from_agent_on_vm_port_deletion(
            self, non_admin_port=False):
        # register l3 agent in dvr mode in addition to existing dvr_snat agent
        HOST = 'host1'
        non_admin_tenant = 'tenant1'
        helpers.register_l3_agent(
            host=HOST, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router()
        with self.network(shared=True) as net,\
                self.subnet(network=net) as subnet,\
                self.port(subnet=subnet,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          tenant_id=non_admin_tenant,
                          set_context=non_admin_port) as port:
            self.core_plugin.update_port(
                    self.context, port['port']['id'],
                    {'port': {portbindings.HOST_ID: HOST}})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})

            with mock.patch.object(self.l3_plugin.l3_rpc_notifier,
                                   'router_removed_from_agent') as remove_mock:
                ctx = context.Context(
                    '', non_admin_tenant) if non_admin_port else self.context
                self._delete('ports', port['port']['id'], neutron_context=ctx)
                remove_mock.assert_called_once_with(
                    mock.ANY, router['id'], HOST)

    def test_router_remove_from_agent_on_vm_port_deletion(self):
        self._test_router_remove_from_agent_on_vm_port_deletion()

    def test_admin_router_remove_from_agent_on_vm_port_deletion(self):
        self._test_router_remove_from_agent_on_vm_port_deletion(
            non_admin_port=True)

    def test_dvr_router_notifications_for_live_migration_with_fip(self):
        self._dvr_router_notifications_for_live_migration(
            with_floatingip=True)

    def test_dvr_router_notifications_for_live_migration_without_fip(self):
        self._dvr_router_notifications_for_live_migration()

    def _dvr_router_notifications_for_live_migration(
            self, with_floatingip=False):
        """Check the router notifications go to the right hosts
        with live migration without hostbinding on the port.
        """
        # register l3 agents in dvr mode in addition to existing dvr_snat agent
        HOST1, HOST2 = 'host1', 'host2'
        for host in [HOST1, HOST2]:
            helpers.register_l3_agent(
                host=host, agent_mode=constants.L3_AGENT_MODE_DVR)

        router = self._create_router()
        arg_list = (portbindings.HOST_ID,)
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as subnet1,\
                self.port(subnet=subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: HOST1}) as vm_port:
            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})
            # add external gateway to router
            self.l3_plugin.update_router(
                self.context, router['id'],
                {'router': {
                    'external_gateway_info': {'network_id': ext_net_id}}})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet1['subnet']['id']})
            if with_floatingip:
                floating_ip = {'floating_network_id': ext_net_id,
                               'router_id': router['id'],
                               'port_id': vm_port['port']['id'],
                               'tenant_id': vm_port['port']['tenant_id'],
                               'dns_name': '', 'dns_domain': ''}
                floating_ip = self.l3_plugin.create_floatingip(
                    self.context, {'floatingip': floating_ip})

            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier,\
                    mock.patch.object(
                        self.l3_plugin,
                        'create_fip_agent_gw_port_if_not_exists'
                                     ) as fip_agent:
                live_migration_port_profile = {
                    'migrating_to': HOST2
                }
                # Update the VM Port with Migration porbinding Profile.
                # With this change, it should trigger a notification to
                # the Destination host to create a Router ahead of time
                # before the VM Port binding has changed to HOST2.
                updated_port = self.core_plugin.update_port(
                    self.context, vm_port['port']['id'],
                    {'port': {
                        portbindings.PROFILE: live_migration_port_profile}})
                # this will be called twice, once for port update, and once
                # for new binding
                l3_notifier.routers_updated_on_host.assert_any_call(
                    self.context, {router['id']}, HOST2)
                # Check the port-binding is still with the old HOST1, but
                # the router update notification has been sent to the new
                # host 'HOST2' based on the live migration profile change.
                self.assertEqual(updated_port[portbindings.HOST_ID], HOST1)
                self.assertNotEqual(updated_port[portbindings.HOST_ID], HOST2)
                if with_floatingip:
                    fip_agent.return_value = True
                    # Since we have already created the floatingip for the
                    # port, it should be creating the floatingip agent gw
                    # port for the new host if it does not exist.
                    fip_agent.assert_any_call(
                        mock.ANY, floating_ip['floating_network_id'], HOST2)

    def test_router_notifications(self):
        """Check that notifications go to the right hosts in different
        conditions
        """
        # register l3 agents in dvr mode in addition to existing dvr_snat agent
        HOST1, HOST2, HOST3 = 'host1', 'host2', 'host3'
        for host in [HOST1, HOST2, HOST3]:
            helpers.register_l3_agent(
                host=host, agent_mode=constants.L3_AGENT_MODE_DVR)

        router = self._create_router()
        arg_list = (portbindings.HOST_ID,)
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as subnet1,\
                self.subnet(cidr='30.0.0.0/24') as subnet2,\
                self.subnet(cidr='40.0.0.0/24') as subnet3,\
                self.port(subnet=subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: HOST1}),\
                self.port(subnet=subnet2,
                          device_owner=constants.DEVICE_OWNER_DHCP,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: HOST2}),\
                self.port(subnet=subnet3,
                          device_owner=constants.DEVICE_OWNER_NEUTRON_PREFIX,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: HOST3}):
            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})

            with mock.patch.object(self.l3_plugin.l3_rpc_notifier.client,
                                   'prepare') as mock_prepare:
                # add external gateway to router
                self.l3_plugin.update_router(
                    self.context, router['id'],
                    {'router': {
                        'external_gateway_info': {'network_id': ext_net_id}}})
                # router has no interfaces so notification goes
                # to only dvr_snat agent
                mock_prepare.assert_called_once_with(
                    server=self.l3_agent['host'],
                    topic=topics.L3_AGENT,
                    version='1.1')

                mock_prepare.reset_mock()
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet1['subnet']['id']})
                self.assertEqual(2, mock_prepare.call_count)
                expected = [mock.call(server=self.l3_agent['host'],
                                      topic=topics.L3_AGENT,
                                      version='1.1'),
                            mock.call(server=HOST1,
                                      topic=topics.L3_AGENT,
                                      version='1.1')]
                mock_prepare.assert_has_calls(expected, any_order=True)

                mock_prepare.reset_mock()
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet2['subnet']['id']})
                self.assertEqual(3, mock_prepare.call_count)
                expected = [mock.call(server=self.l3_agent['host'],
                                      topic=topics.L3_AGENT,
                                      version='1.1'),
                            mock.call(server=HOST1,
                                      topic=topics.L3_AGENT,
                                      version='1.1'),
                            mock.call(server=HOST2,
                                      topic=topics.L3_AGENT,
                                      version='1.1')]
                mock_prepare.assert_has_calls(expected, any_order=True)

                mock_prepare.reset_mock()
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet3['subnet']['id']})
                # there are no dvr serviceable ports on HOST3, so notification
                # goes to the same hosts
                self.assertEqual(3, mock_prepare.call_count)
                expected = [mock.call(server=self.l3_agent['host'],
                                      topic=topics.L3_AGENT,
                                      version='1.1'),
                            mock.call(server=HOST1,
                                      topic=topics.L3_AGENT,
                                      version='1.1'),
                            mock.call(server=HOST2,
                                      topic=topics.L3_AGENT,
                                      version='1.1')]
                mock_prepare.assert_has_calls(expected, any_order=True)

    def test_router_is_not_removed_from_snat_agent_on_interface_removal(self):
        """Check that dvr router is not removed from l3 agent hosting
        SNAT for it on router interface removal
        """
        router = self._create_router()
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        with self.subnet() as subnet,\
                self.network(**kwargs) as ext_net,\
                self.subnet(network=ext_net, cidr='20.0.0.0/24'):
            self.l3_plugin._update_router_gw_info(
                self.context, router['id'],
                {'network_id': ext_net['network']['id']})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})

            agents = self.l3_plugin.list_l3_agents_hosting_router(
                self.context, router['id'])
            self.assertEqual(1, len(agents['agents']))
            with mock.patch.object(self.l3_plugin,
                                   '_l3_rpc_notifier') as l3_notifier:
                self.l3_plugin.remove_router_interface(
                        self.context, router['id'],
                        {'subnet_id': subnet['subnet']['id']})
                agents = self.l3_plugin.list_l3_agents_hosting_router(
                    self.context, router['id'])
                self.assertEqual(1, len(agents['agents']))
                self.assertFalse(l3_notifier.router_removed_from_agent.called)

    def test_router_is_not_removed_from_snat_agent_on_dhcp_port_deletion(self):
        """Check that dvr router is not removed from l3 agent hosting
        SNAT for it on DHCP port removal
        """
        router = self._create_router()
        kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                  extnet_apidef.EXTERNAL: True}
        with self.network(**kwargs) as ext_net,\
                self.subnet(network=ext_net),\
                self.subnet(cidr='20.0.0.0/24') as subnet,\
                self.port(subnet=subnet,
                          device_owner=constants.DEVICE_OWNER_DHCP) as port:
            self.core_plugin.update_port(
                    self.context, port['port']['id'],
                    {'port': {'binding:host_id': self.l3_agent['host']}})
            self.l3_plugin._update_router_gw_info(
                self.context, router['id'],
                {'network_id': ext_net['network']['id']})
            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})

            # router should be scheduled to the dvr_snat l3 agent
            agents = self.l3_plugin.list_l3_agents_hosting_router(
                self.context, router['id'])
            self.assertEqual(1, len(agents['agents']))
            self.assertEqual(self.l3_agent['id'], agents['agents'][0]['id'])

            notifier = self.l3_plugin.agent_notifiers[
                constants.AGENT_TYPE_L3]
            with mock.patch.object(
                    notifier, 'router_removed_from_agent') as remove_mock:
                self._delete('ports', port['port']['id'])
                # now when port is deleted the router still has external
                # gateway and should still be scheduled to the snat agent
                agents = self.l3_plugin.list_l3_agents_hosting_router(
                    self.context, router['id'])
                self.assertEqual(1, len(agents['agents']))
                self.assertEqual(self.l3_agent['id'],
                                 agents['agents'][0]['id'])
                self.assertFalse(remove_mock.called)

    def test__get_dvr_subnet_ids_on_host_query(self):
        with self.subnet(cidr='20.0.0.0/24') as subnet1,\
                self.subnet(cidr='30.0.0.0/24') as subnet2,\
                self.subnet(cidr='40.0.0.0/24') as subnet3,\
                self.port(subnet=subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE) as p1,\
                self.port(subnet=subnet2,
                          device_owner=constants.DEVICE_OWNER_DHCP) as p2,\
                self.port(subnet=subnet3,
                          device_owner=constants.DEVICE_OWNER_NEUTRON_PREFIX)\
                as p3,\
                self.port(subnet=subnet3,
                          device_owner=constants.DEVICE_OWNER_COMPUTE_PREFIX)\
                as p4:
            host = 'host1'

            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual([], subnet_ids)

            self.core_plugin.update_port(
                self.context, p1['port']['id'],
                {'port': {portbindings.HOST_ID: host}})
            expected = {subnet1['subnet']['id']}
            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual(expected, set(subnet_ids))

            self.core_plugin.update_port(
                self.context, p2['port']['id'],
                {'port': {portbindings.HOST_ID: host}})
            expected.add(subnet2['subnet']['id'])
            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual(expected, set(subnet_ids))

            self.core_plugin.update_port(
                self.context, p3['port']['id'],
                {'port': {portbindings.HOST_ID: host}})
            # p3 is non dvr serviceable so no subnet3 expected
            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual(expected, set(subnet_ids))

            other_host = 'other' + host
            self.core_plugin.update_port(
                self.context, p4['port']['id'],
                {'port': {portbindings.HOST_ID: other_host}})
            # p4 is on other host so no subnet3 expected
            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual(expected, set(subnet_ids))

            self.core_plugin.update_port(
                self.context, p4['port']['id'],
                {'port': {portbindings.HOST_ID: host}})
            # finally p4 is on the right host so subnet3 is expected
            expected.add(subnet3['subnet']['id'])
            subnet_ids = [item[0] for item in
                          self.l3_plugin._get_dvr_subnet_ids_on_host_query(
                              self.context, host)]
            self.assertEqual(expected, set(subnet_ids))

    def test__get_dvr_router_ids_for_host(self):
        router1 = self._create_router()
        router2 = self._create_router()
        host = 'host1'
        arg_list = (portbindings.HOST_ID,)
        with self.subnet(cidr='20.0.0.0/24') as subnet1,\
                self.subnet(cidr='30.0.0.0/24') as subnet2,\
                self.port(subnet=subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: host}),\
                self.port(subnet=subnet2,
                          device_owner=constants.DEVICE_OWNER_DHCP,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: host}):

            router_ids = self.l3_plugin._get_dvr_router_ids_for_host(
                self.context, host)
            self.assertEqual([], router_ids)

            self.l3_plugin.add_router_interface(
                self.context, router1['id'],
                {'subnet_id': subnet1['subnet']['id']})
            router_ids = self.l3_plugin._get_dvr_router_ids_for_host(
                self.context, host)
            expected = {router1['id']}
            self.assertEqual(expected, set(router_ids))

            self.l3_plugin.add_router_interface(
                self.context, router2['id'],
                {'subnet_id': subnet2['subnet']['id']})
            router_ids = self.l3_plugin._get_dvr_router_ids_for_host(
                self.context, host)
            expected.add(router2['id'])
            self.assertEqual(expected, set(router_ids))

    def test__get_router_ids_for_agent(self):
        router1 = self._create_router()
        router2 = self._create_router()
        router3 = self._create_router()
        arg_list = (portbindings.HOST_ID,)
        host = self.l3_agent['host']
        with self.subnet() as ext_subnet,\
                self.subnet(cidr='20.0.0.0/24') as subnet1,\
                self.subnet(cidr='30.0.0.0/24') as subnet2,\
                self.port(subnet=subnet1,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: host}),\
                self.port(subnet=subnet2,
                          device_owner=constants.DEVICE_OWNER_DHCP,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: host}):
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [])
            self.assertEqual([], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id'], router2['id']])
            self.assertEqual([], ids)

            self.l3_plugin.add_router_interface(
                self.context, router1['id'],
                {'subnet_id': subnet1['subnet']['id']})
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [])
            self.assertEqual([router1['id']], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id']])
            self.assertEqual([router1['id']], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id'], router2['id']])
            self.assertEqual([router1['id']], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router2['id']])
            self.assertEqual([], ids)

            self.l3_plugin.add_router_interface(
                self.context, router2['id'],
                {'subnet_id': subnet2['subnet']['id']})
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [])
            self.assertEqual({router1['id'], router2['id']}, set(ids))
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id']])
            self.assertEqual([router1['id']], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id'], router2['id']])
            self.assertEqual({router1['id'], router2['id']}, set(ids))
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router2['id']])
            self.assertEqual([router2['id']], ids)

            # make net external
            ext_net_id = ext_subnet['subnet']['network_id']
            self._update('networks', ext_net_id,
                     {'network': {extnet_apidef.EXTERNAL: True}})
            # add external gateway to router
            self.l3_plugin.update_router(
                self.context, router3['id'],
                {'router': {
                    'external_gateway_info': {'network_id': ext_net_id}}})
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [])
            self.assertEqual({router1['id'], router2['id'], router3['id']},
                             set(ids))
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router3['id']])
            self.assertEqual([router3['id']], ids)
            ids = self.l3_plugin._get_router_ids_for_agent(
                self.context, self.l3_agent, [router1['id'], router3['id']])
            self.assertEqual({router1['id'], router3['id']}, set(ids))

    def test_remove_router_interface(self):
        HOST1 = 'host1'
        helpers.register_l3_agent(
            host=HOST1, agent_mode=constants.L3_AGENT_MODE_DVR)
        router = self._create_router()
        arg_list = (portbindings.HOST_ID,)
        with self.subnet() as subnet,\
                self.port(subnet=subnet,
                          device_owner=DEVICE_OWNER_COMPUTE,
                          arg_list=arg_list,
                          **{portbindings.HOST_ID: HOST1}):
            l3_notifier = mock.Mock()
            self.l3_plugin.l3_rpc_notifier = l3_notifier
            self.l3_plugin.agent_notifiers[
                    constants.AGENT_TYPE_L3] = l3_notifier

            self.l3_plugin.add_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})
            self.l3_plugin.schedule_router(self.context, router['id'])

            self.l3_plugin.remove_router_interface(
                self.context, router['id'],
                {'subnet_id': subnet['subnet']['id']})

            l3_notifier.router_removed_from_agent.assert_called_once_with(
                mock.ANY, router['id'], HOST1)

    def test_router_auto_scheduling(self):
        router = self._create_router()
        agents = self.l3_plugin.list_l3_agents_hosting_router(
                self.context, router['id'])
        # router is not scheduled yet
        self.assertEqual([], agents['agents'])

        l3_rpc_handler = l3_rpc.L3RpcCallback()
        # router should be auto scheduled once l3 agent requests router ids
        l3_rpc_handler.get_router_ids(self.context, self.l3_agent['host'])
        agents = self.l3_plugin.list_l3_agents_hosting_router(
            self.context, router['id'])
        self.assertEqual(1, len(agents['agents']))
        self.assertEqual(self.l3_agent['id'], agents['agents'][0]['id'])

    def test_add_router_interface_by_subnet_notifications(self):
        notif_handler_before = mock.Mock()
        notif_handler_after = mock.Mock()
        registry.subscribe(notif_handler_before.callback,
                           resources.ROUTER_INTERFACE,
                           events.BEFORE_CREATE)
        registry.subscribe(notif_handler_after.callback,
                           resources.ROUTER_INTERFACE,
                           events.AFTER_CREATE)
        router = self._create_router()
        with self.network() as net, \
                self.subnet(network=net) as subnet:
            interface_info = {'subnet_id': subnet['subnet']['id']}
            self.l3_plugin.add_router_interface(
                    self.context, router['id'], interface_info)
            kwargs = {'context': self.context, 'router_id': router['id'],
                      'network_id': net['network']['id'],
                      'router_db': mock.ANY,
                      'port': mock.ANY,
                      'interface_info': interface_info}
            notif_handler_before.callback.assert_called_once_with(
                resources.ROUTER_INTERFACE, events.BEFORE_CREATE,
                mock.ANY, **kwargs)
            kwargs_after = {'cidrs': mock.ANY,
                            'context': mock.ANY,
                            'gateway_ips': mock.ANY,
                            'interface_info': mock.ANY,
                            'network_id': None,
                            'port': mock.ANY,
                            'new_interface': True,
                            'subnets': mock.ANY,
                            'port_id': mock.ANY,
                            'router_id': router['id']}
            notif_handler_after.callback.assert_called_once_with(
                resources.ROUTER_INTERFACE, events.AFTER_CREATE,
                mock.ANY, **kwargs_after)

    def test_add_router_interface_by_port_notifications(self):
        notif_handler_before = mock.Mock()
        notif_handler_after = mock.Mock()
        registry.subscribe(notif_handler_before.callback,
                           resources.ROUTER_INTERFACE,
                           events.BEFORE_CREATE)
        registry.subscribe(notif_handler_after.callback,
                           resources.ROUTER_INTERFACE,
                           events.AFTER_CREATE)
        router = self._create_router()
        with self.network() as net, \
                self.subnet(network=net) as subnet, \
                self.port(subnet=subnet) as port:
            interface_info = {'port_id': port['port']['id']}
            self.l3_plugin.add_router_interface(
                    self.context, router['id'], interface_info)
            kwargs = {'context': self.context, 'router_id': router['id'],
                      'network_id': net['network']['id'],
                      'router_db': mock.ANY,
                      'port': mock.ANY,
                      'interface_info': interface_info}
            notif_handler_before.callback.assert_called_once_with(
                resources.ROUTER_INTERFACE, events.BEFORE_CREATE,
                mock.ANY, **kwargs)
            kwargs_after = {'cidrs': mock.ANY,
                            'context': mock.ANY,
                            'gateway_ips': mock.ANY,
                            'interface_info': mock.ANY,
                            'network_id': None,
                            'port': mock.ANY,
                            'new_interface': True,
                            'subnets': mock.ANY,
                            'port_id': port['port']['id'],
                            'router_id': router['id']}
            notif_handler_after.callback.assert_called_once_with(
                resources.ROUTER_INTERFACE, events.AFTER_CREATE,
                mock.ANY, **kwargs_after)


class L3DvrTestCaseMigration(L3DvrTestCaseBase):
    def test_update_router_db_centralized_to_distributed_with_ports(self):
        with self.subnet() as subnet1:
            kwargs = {'arg_list': (extnet_apidef.EXTERNAL,),
                      extnet_apidef.EXTERNAL: True}
            with self.network(**kwargs) as ext_net, \
                    self.subnet(network=ext_net,
                                cidr='30.0.0.0/24'):
                router = self._create_router(distributed=False)
                self.l3_plugin.add_router_interface(
                    self.context, router['id'],
                    {'subnet_id': subnet1['subnet']['id']})
                self.l3_plugin._update_router_gw_info(
                    self.context, router['id'],
                    {'network_id': ext_net['network']['id']})
                self.assertEqual(
                    0, len(self.l3_plugin._get_snat_sync_interfaces(
                        self.context, [router['id']])))

                # router needs to be in admin state down in order to be
                # upgraded to DVR
                self.l3_plugin.update_router(
                    self.context, router['id'],
                    {'router': {'admin_state_up': False}})
                self.assertFalse(router['distributed'])
                self.l3_plugin.update_router(
                    self.context, router['id'],
                    {'router': {'distributed': True}})
                router = self.l3_plugin.get_router(self.context, router['id'])
                self.assertTrue(router['distributed'])
                self.assertEqual(
                    1, len(self.l3_plugin._get_snat_sync_interfaces(
                        self.context, [router['id']])))



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import context
from oslo_utils import uuidutils

from neutron.ipam.drivers.neutrondb_ipam import db_api
from neutron.objects import ipam as ipam_obj
from neutron.tests.unit import testlib_api

CORE_PLUGIN = 'neutron.db.db_base_plugin_v2.NeutronDbPluginV2'


class TestIpamSubnetManager(testlib_api.SqlTestCase):
    """Test case for SubnetManager DB helper class"""

    def setUp(self):
        super(TestIpamSubnetManager, self).setUp()
        self.setup_coreplugin(core_plugin=CORE_PLUGIN)
        self.ctx = context.get_admin_context()
        self.neutron_subnet_id = uuidutils.generate_uuid()
        self.ipam_subnet_id = uuidutils.generate_uuid()
        self.subnet_ip = '1.2.3.4'
        self.single_pool = ('1.2.3.4', '1.2.3.10')
        self.multi_pool = (('1.2.3.2', '1.2.3.12'), ('1.2.3.15', '1.2.3.24'))
        self.subnet_manager = db_api.IpamSubnetManager(self.ipam_subnet_id,
                                                       self.neutron_subnet_id)
        self.subnet_manager_id = self.subnet_manager.create(self.ctx)
        self.ctx.session.flush()

    def test_create(self):
        self.assertEqual(self.ipam_subnet_id, self.subnet_manager_id)
        subnet_count = ipam_obj.IpamSubnet.count(
            self.ctx, id=self.ipam_subnet_id)
        self.assertEqual(1, subnet_count)

    def test_remove(self):
        count = db_api.IpamSubnetManager.delete(self.ctx,
                                                self.neutron_subnet_id)
        self.assertEqual(1, count)
        subnet_exists = ipam_obj.IpamSubnet.objects_exist(
            self.ctx, id=self.ipam_subnet_id)
        self.assertFalse(subnet_exists)

    def test_remove_non_existent_subnet(self):
        count = db_api.IpamSubnetManager.delete(self.ctx,
                                                'non-existent')
        self.assertEqual(0, count)

    def _validate_ips(self, pools, db_pool):
        self.assertTrue(
            any(pool == (str(db_pool.first_ip), str(db_pool.last_ip))
                for pool in pools))

    def test_create_pool(self):
        self.subnet_manager.create_pool(self.ctx,
                                        self.single_pool[0],
                                        self.single_pool[1])

        ipam_pools = ipam_obj.IpamAllocationPool.get_objects(
            self.ctx, ipam_subnet_id=self.ipam_subnet_id)
        self._validate_ips([self.single_pool], ipam_pools[0])

    def test_check_unique_allocation(self):
        self.assertTrue(self.subnet_manager.check_unique_allocation(
            self.ctx, self.subnet_ip))

    def test_check_unique_allocation_negative(self):
        self.subnet_manager.create_allocation(self.ctx,
                                              self.subnet_ip)
        self.assertFalse(self.subnet_manager.check_unique_allocation(
            self.ctx, self.subnet_ip))

    def test_list_allocations(self):
        ips = ['1.2.3.4', '1.2.3.6', '1.2.3.7']
        for ip in ips:
            self.subnet_manager.create_allocation(self.ctx, ip)
        allocs = self.subnet_manager.list_allocations(self.ctx)
        self.assertEqual(len(ips), len(allocs))
        for allocation in allocs:
            self.assertIn(str(allocation.ip_address), ips)

    def _test_create_allocation(self):
        self.subnet_manager.create_allocation(self.ctx,
                                              self.subnet_ip)
        alloc = ipam_obj.IpamAllocation.get_objects(
            self.ctx, ipam_subnet_id=self.ipam_subnet_id)
        self.assertEqual(1, len(alloc))
        self.assertEqual(self.subnet_ip, str(alloc[0].ip_address))
        return alloc

    def test_create_allocation(self):
        self._test_create_allocation()

    def test_delete_allocation(self):
        allocs = self._test_create_allocation()
        self.subnet_manager.delete_allocation(self.ctx,
                                              allocs[0].ip_address)

        alloc_exists = ipam_obj.IpamAllocation.objects_exist(
            self.ctx, ipam_subnet_id=self.ipam_subnet_id)
        self.assertFalse(alloc_exists)



# Copyright (C) 2014,2015 VA Linux Systems Japan K.K.
# Copyright (C) 2014,2015 YAMAMOTO Takashi <yamamoto at valinux co jp>
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import netaddr

import neutron.plugins.ml2.drivers.openvswitch.agent.common.constants \
    as ovs_const
from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent.\
    openflow.ovs_ofctl import ovs_bridge_test_base


call = mock.call  # short hand


class OVSTunnelBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase,
                          ovs_bridge_test_base.OVSDVRProcessTestMixin):
    dvr_process_table_id = ovs_const.DVR_PROCESS
    dvr_process_next_table_id = ovs_const.PATCH_LV_TO_TUN

    def setUp(self):
        super(OVSTunnelBridgeTest, self).setUp()
        self.setup_bridge_mock('br-tun', self.br_tun_cls)
        self.stamp = self.br.default_cookie

    def test_setup_default_table(self):
        patch_int_ofport = 5555
        mock_do_action_flows = mock.patch.object(self.br,
                                                 'do_action_flows').start()
        self.mock.attach_mock(mock_do_action_flows, 'do_action_flows')
        self.br.setup_default_table(patch_int_ofport=patch_int_ofport,
                                    arp_responder_enabled=False)
        flow_args = [{'priority': 1, 'in_port': patch_int_ofport,
                      'actions': 'resubmit(,2)'},
                     {'priority': 0, 'actions': 'drop'},
                     {'priority': 0, 'table': 2,
                      'dl_dst': '00:00:00:00:00:00/01:00:00:00:00:00',
                      'actions': 'resubmit(,20)'},
                     {'priority': 0, 'table': 2,
                      'dl_dst': '01:00:00:00:00:00/01:00:00:00:00:00',
                      'actions': 'resubmit(,22)'},
                     {'priority': 0, 'table': 3, 'actions': 'drop'},
                     {'priority': 0, 'table': 4, 'actions': 'drop'},
                     {'priority': 0, 'table': 6, 'actions': 'drop'},
                     {'priority': 1, 'table': 10,
                      'actions': 'learn(cookie=' + str(self.stamp) +
                      ',table=20,priority=1,hard_timeout=300,'
                      'NXM_OF_VLAN_TCI[0..11],'
                      'NXM_OF_ETH_DST[]=NXM_OF_ETH_SRC[],'
                      'load:0->NXM_OF_VLAN_TCI[],'
                      'load:NXM_NX_TUN_ID[]->NXM_NX_TUN_ID[],'
                      'output:NXM_OF_IN_PORT[]),'
                      'output:%s' % patch_int_ofport},
                     {'priority': 0, 'table': 20, 'actions': 'resubmit(,22)'}
                     ]
        expected = [call.do_action_flows('add', flow_args, False),
                    call.add_flow(priority=0, table=22, actions='drop')]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_setup_default_table_arp_responder_enabled(self):
        patch_int_ofport = 5555
        mock_do_action_flows = mock.patch.object(self.br,
                                                 'do_action_flows').start()
        self.mock.attach_mock(mock_do_action_flows, 'do_action_flows')
        self.br.setup_default_table(patch_int_ofport=patch_int_ofport,
            arp_responder_enabled=True)
        flow_args = [{'priority': 1, 'in_port': patch_int_ofport,
                      'actions': 'resubmit(,2)'},
                     {'priority': 0, 'actions': 'drop'},
                     {'priority': 1, 'table': 2, 'dl_dst': 'ff:ff:ff:ff:ff:ff',
                      'actions': 'resubmit(,21)', 'proto': 'arp'},
                     {'priority': 0, 'table': 2,
                      'dl_dst': '00:00:00:00:00:00/01:00:00:00:00:00',
                      'actions': 'resubmit(,20)'},
                     {'priority': 0, 'table': 2,
                      'dl_dst': '01:00:00:00:00:00/01:00:00:00:00:00',
                      'actions': 'resubmit(,22)'},
                     {'priority': 0, 'table': 3, 'actions': 'drop'},
                     {'priority': 0, 'table': 4, 'actions': 'drop'},
                     {'priority': 0, 'table': 6, 'actions': 'drop'},
                     {'priority': 1, 'table': 10,
                      'actions': 'learn(cookie=' + str(self.stamp) +
                      ',table=20,priority=1,hard_timeout=300,'
                      'NXM_OF_VLAN_TCI[0..11],'
                      'NXM_OF_ETH_DST[]=NXM_OF_ETH_SRC[],'
                      'load:0->NXM_OF_VLAN_TCI[],'
                      'load:NXM_NX_TUN_ID[]->NXM_NX_TUN_ID[],'
                      'output:NXM_OF_IN_PORT[]),'
                      'output:%s' % patch_int_ofport},
                     {'priority': 0, 'table': 20, 'actions': 'resubmit(,22)'},
                     {'priority': 0, 'table': 21, 'actions': 'resubmit(,22)'}
                     ]
        expected = [call.do_action_flows('add', flow_args, False),
                    call.add_flow(priority=0, table=22, actions='drop')]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_provision_local_vlan(self):
        network_type = 'vxlan'
        lvid = 888
        segmentation_id = 777
        distributed = False
        self.br.provision_local_vlan(network_type=network_type, lvid=lvid,
                                     segmentation_id=segmentation_id,
                                     distributed=distributed)
        expected = [
            call.add_flow(priority=1, tun_id=segmentation_id,
                          actions='mod_vlan_vid:%s,resubmit(,10)' % lvid,
                          table=4),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_reclaim_local_vlan(self):
        network_type = 'vxlan'
        segmentation_id = 777
        self.br.reclaim_local_vlan(network_type=network_type,
                                   segmentation_id=segmentation_id)
        expected = [
            call.delete_flows(tun_id=segmentation_id, table=4),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_install_flood_to_tun(self):
        vlan = 3333
        tun_id = 2222
        ports = [11, 44, 22, 33]
        self.br.install_flood_to_tun(vlan=vlan,
                                     tun_id=tun_id,
                                     ports=ports)
        expected = [
            call.mod_flow(table=22, dl_vlan=vlan,
                          actions='strip_vlan,set_tunnel:%(tun)s,'
                          'output:%(ports)s' % {
                              'tun': tun_id,
                              'ports': ','.join(map(str, ports)),
                          }),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_delete_flood_to_tun(self):
        vlan = 3333
        self.br.delete_flood_to_tun(vlan=vlan)
        expected = [
            call.delete_flows(table=22, dl_vlan=vlan),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_install_unicast_to_tun(self):
        vlan = 3333
        port = 55
        mac = '08:60:6e:7f:74:e7'
        tun_id = 2222
        self.br.install_unicast_to_tun(vlan=vlan,
                                       tun_id=tun_id,
                                       port=port,
                                       mac=mac)
        expected = [
            call.add_flow(priority=2, table=20, dl_dst=mac, dl_vlan=vlan,
                          actions='strip_vlan,set_tunnel:%(tun)s,'
                          'output:%(port)s' % {
                              'tun': tun_id,
                              'port': port,
                          }),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_delete_unicast_to_tun(self):
        vlan = 3333
        mac = '08:60:6e:7f:74:e7'
        self.br.delete_unicast_to_tun(vlan=vlan, mac=mac)
        expected = [
            call.delete_flows(table=20, dl_dst=mac, dl_vlan=vlan),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_delete_unicast_to_tun_without_mac(self):
        vlan = 3333
        mac = None
        self.br.delete_unicast_to_tun(vlan=vlan, mac=mac)
        expected = [
            call.delete_flows(table=20, dl_vlan=vlan),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_install_arp_responder(self):
        vlan = 3333
        ip = '192.0.2.1'
        mac = '08:60:6e:7f:74:e7'
        self.br.install_arp_responder(vlan=vlan, ip=ip, mac=mac)
        expected = [
            call.add_flow(proto='arp', nw_dst=ip,
                          actions='move:NXM_OF_ETH_SRC[]->NXM_OF_ETH_DST[],'
                          'mod_dl_src:%(mac)s,load:0x2->NXM_OF_ARP_OP[],'
                          'move:NXM_NX_ARP_SHA[]->NXM_NX_ARP_THA[],'
                          'move:NXM_OF_ARP_SPA[]->NXM_OF_ARP_TPA[],'
                          'load:%(mac)#x->NXM_NX_ARP_SHA[],'
                          'load:%(ip)#x->NXM_OF_ARP_SPA[],in_port' % {
                              'mac': netaddr.EUI(mac,
                                                 dialect=netaddr.mac_unix),
                              'ip': netaddr.IPAddress(ip),
                          },
                          priority=1, table=21, dl_vlan=vlan),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_delete_arp_responder(self):
        vlan = 3333
        ip = '192.0.2.1'
        self.br.delete_arp_responder(vlan=vlan, ip=ip)
        expected = [
            call.delete_flows(table=21, dl_vlan=vlan, proto='arp', nw_dst=ip),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_delete_arp_responder_without_ip(self):
        vlan = 3333
        ip = None
        self.br.delete_arp_responder(vlan=vlan, ip=ip)
        expected = [
            call.delete_flows(table=21, dl_vlan=vlan, proto='arp'),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_setup_tunnel_port(self):
        network_type = 'vxlan'
        port = 11111
        self.br.setup_tunnel_port(network_type=network_type, port=port)
        expected = [
            call.add_flow(priority=1, in_port=port, actions='resubmit(,4)'),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_cleanup_tunnel_port(self):
        port = 11111
        self.br.cleanup_tunnel_port(port=port)
        expected = [
            call.delete_flows(in_port=port),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_add_dvr_mac_tun(self):
        mac = '00:02:b3:13:fe:3d'
        port = 8888
        self.br.add_dvr_mac_tun(mac=mac, port=port)
        expected = [
            call.add_flow(priority=1, table=9, dl_src=mac,
                          actions='output:%s' % port),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def test_remove_dvr_mac_tun(self):
        mac = '00:02:b3:13:fe:3d'
        self.br.remove_dvr_mac_tun(mac=mac)
        expected = [
            call.delete_flows(dl_src=mac, table=9),
        ]
        self.assertEqual(expected, self.mock.mock_calls)

    def _mock_add_tunnel_port(self, deferred_br=False):
        port_name = 'fake_port'
        remote_ip = '192.168.1.3'
        local_ip = '192.168.1.2'
        tunnel_type = 'vxlan'
        vxlan_udp_port = '4789'
        dont_fragment = True
        if deferred_br:
            with mock.patch('neutron.agent.common.ovs_lib.OVSBridge.add_port',
                            return_value=9999) as add_port, \
                    self.br.deferred() as deferred_br:
                ofport = deferred_br.add_tunnel_port(port_name, remote_ip,
                                                     local_ip, tunnel_type,
                                                     vxlan_udp_port,
                                                     dont_fragment)
        else:
            with mock.patch('neutron.agent.common.ovs_lib.OVSBridge.add_port',
                            return_value=9999) as add_port:
                ofport = self.br.add_tunnel_port(port_name, remote_ip,
                                                 local_ip, tunnel_type,
                                                 vxlan_udp_port,
                                                 dont_fragment)
        self.assertEqual(9999, ofport)
        self.assertEqual(1, add_port.call_count)
        self.assertEqual(port_name, add_port.call_args[0][0])

    def _mock_delete_port(self, deferred_br=False):
        port_name = 'fake_port'
        if deferred_br:
            with mock.patch('neutron.agent.common.ovs_lib.OVSBridge.'
                            'delete_port') as delete_port, \
                    self.br.deferred() as deferred_br:
                deferred_br.delete_port(port_name)
        else:
            with mock.patch('neutron.agent.common.ovs_lib.OVSBridge.'
                            'delete_port') as delete_port:
                self.br.delete_port(port_name)
        self.assertEqual([call(port_name)], delete_port.mock_calls)

    def test_add_tunnel_port(self):
        self._mock_add_tunnel_port()

    def test_delete_port(self):
        self._mock_delete_port()

    def test_deferred_br_add_tunnel_port(self):
        self._mock_add_tunnel_port(True)

    def test_deferred_br_delete_port(self):
        self._mock_delete_port(True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
import pkg_resources

from neutron._i18n import _


MIGRATION_ENTRYPOINTS = 'neutron.db.alembic_migrations'
migration_entrypoints = {
    entrypoint.name: entrypoint
    for entrypoint in pkg_resources.iter_entry_points(MIGRATION_ENTRYPOINTS)
}

INSTALLED_SUBPROJECTS = [project_ for project_ in migration_entrypoints]

CORE_OPTS = [
    cfg.StrOpt('subproject',
               choices=INSTALLED_SUBPROJECTS,
               help=(_("The subproject to execute the command against. "
                       "Can be one of: '%s'.")
                     % "', '".join(INSTALLED_SUBPROJECTS))),
    cfg.BoolOpt('split_branches',
                default=True,
                deprecated_for_removal=True,
                help=_("DEPRECATED in newton, will be removed in ocata."
                       "Alembic environments integrating with "
                       "Neutron must implement split (contract and expand) "
                       "branches file structure."))
]

DB_OPTS = [
    cfg.StrOpt('connection',
               default='',
               secret=True,
               help=_('URL to database')),
    cfg.StrOpt('engine',
               default='',
               help=_('Database engine for which script will be generated '
                      'when using offline migration.')),
]


def register_db_cli_opts(conf):
    conf.register_cli_opts(CORE_OPTS)
    conf.register_cli_opts(DB_OPTS, 'database')



# Copyright 2016 IBM
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""Add segment_host_mapping table.

Revision ID: 8fd3918ef6f4
Revises: c879c5e1ee90
Create Date: 2016-02-25 00:22:47.618593

"""

# revision identifiers, used by Alembic.
revision = '8fd3918ef6f4'
down_revision = 'c879c5e1ee90'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.create_table('segmenthostmappings',
                    sa.Column('segment_id',
                              sa.String(length=36),
                              index=True,
                              nullable=False),
                    sa.Column('host',
                              sa.String(255),
                              index=True,
                              nullable=False),
                    sa.PrimaryKeyConstraint('segment_id', 'host'),
                    sa.ForeignKeyConstraint(['segment_id'],
                                            ['networksegments.id'],
                                            ondelete='CASCADE'))


def contract_creation_exceptions():
    """
    Return create exceptions.

    These elements depend on the networksegments table which was renamed
    in the contract branch.
    """
    return {
        sa.Table: ['segmenthostmappings'],
        sa.Index: ['segmenthostmappings']
    }



# Copyright (c) 2017-18 NEC Technologies India Pvt Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.services.qos import constants as qos_consts

from neutron.common import exceptions as n_exc


def check_bandwidth_rule_conflict(policy, rule_data):
    """Implementation of the QoS Rule checker.

    This function checks if the new rule to be associated with the policy
    doesn't conflict with the existing rules.
    Raises an exception if conflict is identified.
    """
    for rule in policy.rules:
        if rule.rule_type == qos_consts.RULE_TYPE_DSCP_MARKING:
            # Skip checks if Rule is DSCP
            continue
        elif rule.rule_type == qos_consts.RULE_TYPE_MINIMUM_BANDWIDTH:
            if "max_kbps" in rule_data and (
                    int(rule.min_kbps) > int(rule_data["max_kbps"])):
                raise n_exc.QoSRuleParameterConflict(
                    rule_value=rule_data["max_kbps"],
                    policy_id=policy["id"],
                    existing_rule=rule.rule_type,
                    existing_value=rule.min_kbps)
        elif rule.rule_type == qos_consts.RULE_TYPE_BANDWIDTH_LIMIT:
            if "min_kbps" in rule_data and (
                    int(rule.max_kbps) < int(rule_data["min_kbps"])):
                raise n_exc.QoSRuleParameterConflict(
                    rule_value=rule_data["min_kbps"],
                    policy_id=policy["id"],
                    existing_rule=rule.rule_type,
                    existing_value=rule.max_kbps)


def check_rules_conflict(policy, rule_obj):
    """Implementation of the QoS Policy rules conflicts.

    This function checks if the new rule to be associated with policy
    doesn't have any duplicate rule already in policy.
    Raises an exception if conflict is identified.
    """

    for rule in policy.rules:
        # NOTE(slaweq): we don't want to raise exception when compared rules
        # have got same id as it means that it is probably exactly the same
        # rule so there is no conflict
        if rule.id == getattr(rule_obj, "id", None):
            continue
        if rule.duplicates(rule_obj):
            raise n_exc.QoSRulesConflict(
                new_rule_type=rule_obj.rule_type,
                rule_id=rule.id,
                policy_id=policy.id)



# Copyright (c) 2014 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import shutil
import tempfile

import netaddr
from neutron_lib import constants as n_consts
from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import uuidutils

from neutron.agent.common import ovs_lib
from neutron.agent.l3 import ha_router
from neutron.agent.l3 import namespaces
from neutron.agent.linux import external_process
from neutron.agent.linux import ip_lib
from neutron.agent.linux import ip_link_support
from neutron.agent.linux import keepalived
from neutron.agent.linux import utils as agent_utils
from neutron.cmd import runtime_checks
from neutron.common import constants
from neutron.common import utils as common_utils
from neutron.plugins.ml2.drivers.openvswitch.agent.common \
    import constants as ovs_const

LOG = logging.getLogger(__name__)


MINIMUM_DNSMASQ_VERSION = 2.67
DNSMASQ_VERSION_DHCP_RELEASE6 = 2.76
MINIMUM_DIBBLER_VERSION = '1.0.1'


def ovs_vxlan_supported(from_ip='192.0.2.1', to_ip='192.0.2.2'):
    name = common_utils.get_rand_device_name(prefix='vxlantest-')
    with ovs_lib.OVSBridge(name) as br:
        port = br.add_tunnel_port(from_ip, to_ip, n_consts.TYPE_VXLAN)
        return port != ovs_lib.INVALID_OFPORT


def ovs_geneve_supported(from_ip='192.0.2.3', to_ip='192.0.2.4'):
    name = common_utils.get_rand_device_name(prefix='genevetest-')
    with ovs_lib.OVSBridge(name) as br:
        port = br.add_tunnel_port(from_ip, to_ip, n_consts.TYPE_GENEVE)
        return port != ovs_lib.INVALID_OFPORT


def iproute2_vxlan_supported():
    ip = ip_lib.IPWrapper()
    name = common_utils.get_rand_device_name(prefix='vxlantest-')
    port = ip.add_vxlan(name, 3000)
    ip.del_veth(name)
    return name == port.name


def patch_supported():
    name, peer_name, patch_name = common_utils.get_related_rand_device_names(
        ['patchtest-', 'peertest0-', 'peertest1-'])
    with ovs_lib.OVSBridge(name) as br:
        port = br.add_patch_port(patch_name, peer_name)
        return port != ovs_lib.INVALID_OFPORT


def nova_notify_supported():
    try:
        import neutron.notifiers.nova  # noqa since unused
        return True
    except ImportError:
        return False


def ofctl_arg_supported(cmd, **kwargs):
    """Verify if ovs-ofctl binary supports cmd with **kwargs.

    :param cmd: ovs-ofctl command to use for test.
    :param **kwargs: arguments to test with the command.
    :returns: a boolean if the supplied arguments are supported.
    """
    br_name = common_utils.get_rand_device_name(prefix='br-test-')
    with ovs_lib.OVSBridge(br_name) as test_br:
        full_args = ["ovs-ofctl", cmd, test_br.br_name,
                     ovs_lib._build_flow_expr_str(kwargs, cmd.split('-')[0],
                                                  False)]
        try:
            agent_utils.execute(full_args, run_as_root=True)
        except RuntimeError as e:
            LOG.debug("Exception while checking supported feature via "
                      "command %s. Exception: %s", full_args, e)
            return False
        except Exception:
            LOG.exception("Unexpected exception while checking supported"
                          " feature via command: %s", full_args)
            return False
        else:
            return True


def arp_responder_supported():
    mac = netaddr.EUI('dead:1234:beef', dialect=netaddr.mac_unix)
    ip = netaddr.IPAddress('240.0.0.1')
    actions = ovs_const.ARP_RESPONDER_ACTIONS % {'mac': mac, 'ip': ip}

    return ofctl_arg_supported(cmd='add-flow',
                               table=21,
                               priority=1,
                               proto='arp',
                               dl_vlan=42,
                               nw_dst='%s' % ip,
                               actions=actions)


def arp_header_match_supported():
    return ofctl_arg_supported(cmd='add-flow',
                               table=24,
                               priority=1,
                               proto='arp',
                               arp_op='0x2',
                               arp_spa='1.1.1.1',
                               actions="NORMAL")


def icmpv6_header_match_supported():
    return ofctl_arg_supported(cmd='add-flow',
                               table=ovs_const.ARP_SPOOF_TABLE,
                               priority=1,
                               dl_type=constants.ETHERTYPE_IPV6,
                               nw_proto=n_consts.PROTO_NUM_IPV6_ICMP,
                               icmp_type=n_consts.ICMPV6_TYPE_NA,
                               nd_target='fdf8:f53b:82e4::10',
                               actions="NORMAL")


def _vf_management_support(required_caps):
    is_supported = True
    try:
        vf_section = ip_link_support.IpLinkSupport.get_vf_mgmt_section()
        for cap in required_caps:
            if not ip_link_support.IpLinkSupport.vf_mgmt_capability_supported(
                   vf_section, cap):
                is_supported = False
                LOG.debug("ip link command does not support "
                          "vf capability '%(cap)s'", {'cap': cap})
    except ip_link_support.UnsupportedIpLinkCommand:
        LOG.exception("Unexpected exception while checking supported "
                      "ip link command")
        return False
    return is_supported


def vf_management_supported():
    required_caps = (
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_STATE,
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_SPOOFCHK,
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_RATE)
    return _vf_management_support(required_caps)


def vf_extended_management_supported():
    required_caps = (
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_STATE,
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_SPOOFCHK,
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_RATE,
        ip_link_support.IpLinkConstants.IP_LINK_CAPABILITY_MIN_TX_RATE,
    )
    return _vf_management_support(required_caps)


def netns_read_requires_helper():
    nsname = "netnsreadtest-" + uuidutils.generate_uuid()
    ip_lib.create_network_namespace(nsname)
    try:
        # read without root_helper. if exists, not required.
        exists = ip_lib.network_namespace_exists(nsname)
    finally:
        ip_lib.delete_network_namespace(nsname)
    return not exists


def get_minimal_dnsmasq_version_supported():
    return MINIMUM_DNSMASQ_VERSION


def get_dnsmasq_version_with_dhcp_release6():
    return DNSMASQ_VERSION_DHCP_RELEASE6


def dnsmasq_local_service_supported():
    cmd = ['dnsmasq', '--test', '--local-service']
    env = {'LC_ALL': 'C'}
    obj, cmd = agent_utils.create_process(cmd, addl_env=env)
    _stdout, _stderr = obj.communicate()
    returncode = obj.returncode
    if returncode == 127:
        LOG.debug("Exception while checking dnsmasq version. "
                  "dnsmasq: No such file or directory")
        return False
    elif returncode == 1:
        return False
    return True


def dnsmasq_version_supported():
    try:
        cmd = ['dnsmasq', '--version']
        env = {'LC_ALL': 'C'}
        out = agent_utils.execute(cmd, addl_env=env)
        m = re.search(r"version (\d+\.\d+)", out)
        ver = float(m.group(1)) if m else 0
        if ver < MINIMUM_DNSMASQ_VERSION:
            return False
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking minimal dnsmasq version. "
                  "Exception: %s", e)
        return False
    return True


def dhcp_release6_supported():
    return runtime_checks.dhcp_release6_supported()


def bridge_firewalling_enabled():
    for proto in ('arp', 'ip', 'ip6'):
        knob = 'net.bridge.bridge-nf-call-%stables' % proto
        cmd = ['sysctl', '-b', knob]
        try:
            out = agent_utils.execute(cmd)
        except (OSError, RuntimeError, IndexError, ValueError) as e:
            LOG.debug("Exception while extracting %(knob)s. "
                      "Exception: %(e)s", {'knob': knob, 'e': e})
            return False
        if out == '0':
            return False
    return True


class KeepalivedIPv6Test(object):
    def __init__(self, ha_port, gw_port, gw_vip, default_gw):
        self.ha_port = ha_port
        self.gw_port = gw_port
        self.gw_vip = gw_vip
        self.default_gw = default_gw
        self.manager = None
        self.config = None
        self.config_path = None
        self.nsname = "keepalivedtest-" + uuidutils.generate_uuid()
        self.pm = None
        self.orig_interval = cfg.CONF.AGENT.check_child_processes_interval

    def configure(self):
        config = keepalived.KeepalivedConf()
        instance1 = keepalived.KeepalivedInstance('MASTER', self.ha_port, 1,
                                                  ['169.254.192.0/18'],
                                                  advert_int=5)
        instance1.track_interfaces.append(self.ha_port)

        # Configure keepalived with an IPv6 address (gw_vip) on gw_port.
        vip_addr1 = keepalived.KeepalivedVipAddress(self.gw_vip, self.gw_port)
        instance1.vips.append(vip_addr1)

        # Configure keepalived with an IPv6 default route on gw_port.
        gateway_route = keepalived.KeepalivedVirtualRoute(n_consts.IPv6_ANY,
                                                          self.default_gw,
                                                          self.gw_port)
        instance1.virtual_routes.gateway_routes = [gateway_route]
        config.add_instance(instance1)
        self.config = config

    def start_keepalived_process(self):
        # Disable process monitoring for Keepalived process.
        cfg.CONF.set_override('check_child_processes_interval', 0, 'AGENT')
        self.pm = external_process.ProcessMonitor(cfg.CONF, 'router')

        # Create a temp directory to store keepalived configuration.
        self.config_path = tempfile.mkdtemp()

        # Instantiate keepalived manager with the IPv6 configuration.
        self.manager = keepalived.KeepalivedManager('router1', self.config,
            namespace=self.nsname, process_monitor=self.pm,
            conf_path=self.config_path)
        self.manager.spawn()

    def verify_ipv6_address_assignment(self, gw_dev):
        process = self.manager.get_process()
        common_utils.wait_until_true(lambda: process.active)

        def _gw_vip_assigned():
            iface_ip = gw_dev.addr.list(ip_version=6, scope='global')
            if iface_ip:
                return self.gw_vip == iface_ip[0]['cidr']

        common_utils.wait_until_true(_gw_vip_assigned)

    def __enter__(self):
        ip_lib.create_network_namespace(self.nsname)
        return self

    def __exit__(self, exc_type, exc_value, exc_tb):
        if self.pm:
            self.pm.stop()
        if self.manager:
            self.manager.disable()
        if self.config_path:
            shutil.rmtree(self.config_path, ignore_errors=True)
        ip_lib.delete_network_namespace(self.nsname)
        cfg.CONF.set_override('check_child_processes_interval',
                              self.orig_interval, 'AGENT')


def keepalived_ipv6_supported():
    """Check if keepalived supports IPv6 functionality.

    Validation is done as follows.
    1. Create a namespace.
    2. Create OVS bridge with two ports (ha_port and gw_port)
    3. Move the ovs ports to the namespace.
    4. Spawn keepalived process inside the namespace with IPv6 configuration.
    5. Verify if IPv6 address is assigned to gw_port.
    6. Verify if IPv6 default route is configured by keepalived.
    """

    br_name, ha_port, gw_port = common_utils.get_related_rand_device_names(
        ['ka-test-', ha_router.HA_DEV_PREFIX, namespaces.INTERNAL_DEV_PREFIX])
    gw_vip = 'fdf8:f53b:82e4::10/64'
    expected_default_gw = 'fe80:f816::1'

    with ovs_lib.OVSBridge(br_name) as br:
        with KeepalivedIPv6Test(ha_port, gw_port, gw_vip,
                                expected_default_gw) as ka:
            br.add_port(ha_port, ('type', 'internal'))
            br.add_port(gw_port, ('type', 'internal'))

            ha_dev = ip_lib.IPDevice(ha_port)
            gw_dev = ip_lib.IPDevice(gw_port)

            ha_dev.link.set_netns(ka.nsname)
            gw_dev.link.set_netns(ka.nsname)

            ha_dev.link.set_up()
            gw_dev.link.set_up()
            ha_dev.addr.add('169.254.192.8/18')

            ka.configure()

            ka.start_keepalived_process()

            ka.verify_ipv6_address_assignment(gw_dev)

            default_gw = gw_dev.route.get_gateway(ip_version=6)
            if default_gw:
                default_gw = default_gw['gateway']

    return expected_default_gw == default_gw


def ovsdb_native_supported():
    # Running the test should ensure we are configured for OVSDB native
    try:
        ovs = ovs_lib.BaseOVS()
        ovs.get_bridges()
        return True
    except ImportError as ex:
        LOG.error("Failed to import required modules. Ensure that the "
                  "python-openvswitch package is installed. Error: %s",
                  ex)
    except Exception:
        LOG.exception("Unexpected exception occurred.")

    return False


def ovs_conntrack_supported():
    br_name = common_utils.get_rand_device_name(prefix="ovs-test-")

    with ovs_lib.OVSBridge(br_name) as br:
        try:
            br.add_protocols(*["OpenFlow%d" % i for i in range(10, 15)])
        except RuntimeError as e:
            LOG.debug("Exception while checking ovs conntrack support: %s", e)
            return False
    return ofctl_arg_supported(cmd='add-flow', ct_state='+trk', actions='drop')


def ebtables_supported():
    try:
        cmd = ['ebtables', '--version']
        agent_utils.execute(cmd)
        return True
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking for installed ebtables. "
                  "Exception: %s", e)
        return False


def ipset_supported():
    try:
        cmd = ['ipset', '--version']
        agent_utils.execute(cmd)
        return True
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking for installed ipset. "
                  "Exception: %s", e)
        return False


def ip6tables_supported():
    try:
        cmd = ['ip6tables', '--version']
        agent_utils.execute(cmd)
        return True
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking for installed ip6tables. "
                  "Exception: %s", e)
        return False


def conntrack_supported():
    try:
        cmd = ['conntrack', '--version']
        agent_utils.execute(cmd)
        return True
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking for installed conntrack. "
                  "Exception: %s", e)
        return False


def get_minimal_dibbler_version_supported():
    return MINIMUM_DIBBLER_VERSION


def dibbler_version_supported():
    try:
        cmd = ['dibbler-client',
               'help']
        out = agent_utils.execute(cmd)
        return '-w' in out
    except (OSError, RuntimeError, IndexError, ValueError) as e:
        LOG.debug("Exception while checking minimal dibbler version. "
                  "Exception: %s", e)
        return False


def _fix_ip_nonlocal_bind_root_value(original_value):
    current_value = ip_lib.get_ip_nonlocal_bind(namespace=None)
    if current_value != original_value:
        ip_lib.set_ip_nonlocal_bind(value=original_value, namespace=None)


def ip_nonlocal_bind():
    nsname1 = "ipnonlocalbind1-" + uuidutils.generate_uuid()
    nsname2 = "ipnonlocalbind2-" + uuidutils.generate_uuid()

    ip_lib.create_network_namespace(nsname1)
    try:
        ip_lib.create_network_namespace(nsname2)
        try:
            original_value = ip_lib.get_ip_nonlocal_bind(namespace=None)
            try:
                ip_lib.set_ip_nonlocal_bind(value=0, namespace=nsname1)
                ip_lib.set_ip_nonlocal_bind(value=1, namespace=nsname2)
                ns1_value = ip_lib.get_ip_nonlocal_bind(namespace=nsname1)
            finally:
                _fix_ip_nonlocal_bind_root_value(original_value)
        except RuntimeError as e:
            LOG.debug("Exception while checking ip_nonlocal_bind. "
                      "Exception: %s", e)
            return False
        finally:
            ip_lib.delete_network_namespace(nsname2)
    finally:
        ip_lib.delete_network_namespace(nsname1)
    return ns1_value == 0



# Copyright (c) 2014 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.utils import host
from oslo_config import cfg

from neutron._i18n import _
from neutron.agent.linux import keepalived


OPTS = [
    cfg.StrOpt('ha_confs_path',
               default='$state_path/ha_confs',
               help=_('Location to store keepalived/conntrackd '
                      'config files')),
    cfg.StrOpt('ha_vrrp_auth_type',
               default='PASS',
               choices=keepalived.VALID_AUTH_TYPES,
               help=_('VRRP authentication type')),
    cfg.StrOpt('ha_vrrp_auth_password',
               help=_('VRRP authentication password'),
               secret=True),
    cfg.IntOpt('ha_vrrp_advert_int',
               default=2,
               help=_('The advertisement interval in seconds')),
    cfg.IntOpt('ha_keepalived_state_change_server_threads',
               default=(1 + host.cpu_count()) // 2,
               sample_default='(1 + <num_of_cpus>) / 2',
               min=1,
               help=_('Number of concurrent threads for '
                      'keepalived server connection requests. '
                      'More threads create a higher CPU load '
                      'on the agent node.')),
    cfg.IntOpt('ha_vrrp_health_check_interval',
               default=0,
               help=_('The VRRP health check interval in seconds. Values > 0 '
                      'enable VRRP health checks. Setting it to 0 disables '
                      'VRRP health checks. Recommended value is 5. '
                      'This will cause pings to be sent to the gateway '
                      'IP address(es) - requires ICMP_ECHO_REQUEST '
                      'to be enabled on the gateway. '
                      'If gateway fails, all routers will be reported '
                      'as master, and master election will be repeated '
                      'in round-robin fashion, until one of the router '
                      'restore the gateway connection.')),
]


def register_l3_agent_ha_opts(cfg=cfg.CONF):
    cfg.register_opts(OPTS)



# Copyright 2015 Intel Corporation.
# Copyright 2015 Isaku Yamahata <isaku.yamahata at intel com>
#                               <isaku.yamahata at gmail com>
# All Rights Reserved.
#
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import validators
from neutron_lib import constants
from neutron_lib.db import model_base
from neutron_lib.plugins.ml2 import api
import oslo_db.sqlalchemy.session
import sqlalchemy as sa
from sqlalchemy import orm

from neutron.api import extensions
from neutron.db import models_v2
from neutron.objects import subnet as subnet_obj
from neutron.tests.unit.plugins.ml2 import extensions as test_extensions


class TestExtensionDriverBase(api.ExtensionDriver):
    _supported_extension_aliases = 'fake_extension'

    def initialize(self):
        extensions.append_api_extensions_path(test_extensions.__path__)

    @property
    def extension_alias(self):
        return self._supported_extension_aliases


class TestExtensionDriver(TestExtensionDriverBase):
    def initialize(self):
        super(TestExtensionDriver, self).initialize()
        # keep track of values
        self.val_by_id = {}

    def _check_create(self, session, data, result):
        assert(isinstance(session, oslo_db.sqlalchemy.session.Session))
        assert(isinstance(data, dict))
        assert('id' not in data)
        assert(isinstance(result, dict))
        assert(result['id'] is not None)

    def _check_update(self, session, data, result):
        assert(isinstance(session, oslo_db.sqlalchemy.session.Session))
        assert(isinstance(data, dict))
        assert(isinstance(result, dict))
        assert(result['id'] is not None)

    def _check_extend(self, session, result, entry,
                      expected_db_entry_class, expected_obj_entry_class=None):
        # TODO(slaweq): After converting all code to use Subnet OVO,
        # expected_db_entry_class can be removed as only OVO object
        # should be expected here
        assert(isinstance(session, oslo_db.sqlalchemy.session.Session))
        assert(isinstance(result, dict))
        assert(result['id'] is not None)
        if expected_obj_entry_class:
            assert(isinstance(entry, expected_db_entry_class) or
                   isinstance(entry, expected_obj_entry_class))
        else:
            assert(isinstance(entry, expected_db_entry_class))
        assert(entry.id == result['id'])

    def _store_change(self, result, data, field):
        if field in data and data[field] != constants.ATTR_NOT_SPECIFIED:
            self.val_by_id[result['id']] = data[field]
        elif result['id'] not in self.val_by_id:
            self.val_by_id[result['id']] = 'default_%s' % field

    def process_create_network(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_create(session, data, result)
        self._store_change(result, data, 'network_extension')
        result['network_extension'] = self.val_by_id[result['id']]

    def process_update_network(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_update(session, data, result)
        self._store_change(result, data, 'network_extension')
        result['network_extension'] = self.val_by_id[result['id']]

    def extend_network_dict(self, session, net_db, result):
        self._check_extend(session, result, net_db, models_v2.Network)
        result['network_extension'] = self.val_by_id.get(result['id'])

    def process_create_subnet(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_create(session, data, result)
        self._store_change(result, data, 'subnet_extension')
        result['subnet_extension'] = self.val_by_id[result['id']]

    def process_update_subnet(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_update(session, data, result)
        self._store_change(result, data, 'subnet_extension')
        result['subnet_extension'] = self.val_by_id[result['id']]

    def extend_subnet_dict(self, session, subnet_db, result):
        self._check_extend(
            session, result, subnet_db,
            expected_db_entry_class=models_v2.Subnet,
            expected_obj_entry_class=subnet_obj.Subnet)
        result['subnet_extension'] = self.val_by_id.get(result['id'])

    def process_create_port(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_create(session, data, result)
        self._store_change(result, data, 'port_extension')
        result['port_extension'] = self.val_by_id[result['id']]

    def process_update_port(self, plugin_context, data, result):
        session = plugin_context.session
        self._check_update(session, data, result)
        self._store_change(result, data, 'port_extension')
        result['port_extension'] = self.val_by_id[result['id']]

    def extend_port_dict(self, session, port_db, result):
        self._check_extend(session, result, port_db, models_v2.Port)
        result['port_extension'] = self.val_by_id.get(result['id'])


class TestNetworkExtension(model_base.BASEV2):
    network_id = sa.Column(sa.String(36),
                           sa.ForeignKey('networks.id', ondelete="CASCADE"),
                           primary_key=True)
    value = sa.Column(sa.String(64))
    network = orm.relationship(
        models_v2.Network,
        backref=orm.backref('extension', cascade='delete', uselist=False,
                            lazy='joined'))


class TestSubnetExtension(model_base.BASEV2):
    subnet_id = sa.Column(sa.String(36),
                          sa.ForeignKey('subnets.id', ondelete="CASCADE"),
                          primary_key=True)
    value = sa.Column(sa.String(64))
    subnet = orm.relationship(
        models_v2.Subnet,
        backref=orm.backref('extension', cascade='delete', uselist=False,
                            lazy='joined'))


class TestPortExtension(model_base.BASEV2):
    port_id = sa.Column(sa.String(36),
                        sa.ForeignKey('ports.id', ondelete="CASCADE"),
                        primary_key=True)
    value = sa.Column(sa.String(64))
    port = orm.relationship(
        models_v2.Port,
        backref=orm.backref('extension', cascade='delete', uselist=False,
                            lazy='joined'))


class TestDBExtensionDriver(TestExtensionDriverBase):
    def _get_value(self, data, key):
        value = data[key]
        if not validators.is_attr_set(value):
            value = ''
        return value

    def process_create_network(self, plugin_context, data, result):
        session = plugin_context.session
        value = self._get_value(data, 'network_extension')
        record = TestNetworkExtension(network_id=result['id'], value=value)
        session.add(record)
        result['network_extension'] = value

    def process_update_network(self, plugin_context, data, result):
        session = plugin_context.session
        record = (session.query(TestNetworkExtension).
                  filter_by(network_id=result['id']).one())
        value = data.get('network_extension')
        if value and value != record.value:
            record.value = value
        result['network_extension'] = record.value

    def extend_network_dict(self, session, net_db, result):
        result['network_extension'] = net_db.extension.value

    def process_create_subnet(self, plugin_context, data, result):
        session = plugin_context.session
        value = self._get_value(data, 'subnet_extension')
        record = TestSubnetExtension(subnet_id=result['id'], value=value)
        session.add(record)
        result['subnet_extension'] = value

    def process_update_subnet(self, plugin_context, data, result):
        session = plugin_context.session
        record = (session.query(TestSubnetExtension).
                  filter_by(subnet_id=result['id']).one())
        value = data.get('subnet_extension')
        if value and value != record.value:
            record.value = value
        result['subnet_extension'] = record.value

    def extend_subnet_dict(self, session, subnet_db, result):
        value = subnet_db.extension.value if subnet_db.extension else ''
        result['subnet_extension'] = value

    def process_create_port(self, plugin_context, data, result):
        session = plugin_context.session
        value = self._get_value(data, 'port_extension')
        record = TestPortExtension(port_id=result['id'], value=value)
        session.add(record)
        result['port_extension'] = value

    def process_update_port(self, plugin_context, data, result):
        session = plugin_context.session
        record = (session.query(TestPortExtension).
                  filter_by(port_id=result['id']).one())
        value = data.get('port_extension')
        if value and value != record.value:
            record.value = value
        result['port_extension'] = record.value

    def extend_port_dict(self, session, port_db, result):
        value = port_db.extension.value if port_db.extension else ''
        result['port_extension'] = value



# Copyright (c) 2015 Mirantis, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.api import faults
import oslo_i18n
from oslo_log import log as logging
from pecan import hooks

from neutron.api import api_common


LOG = logging.getLogger(__name__)


class ExceptionTranslationHook(hooks.PecanHook):
    def on_error(self, state, e):
        language = None
        if state.request.accept_language:
            language = state.request.accept_language.best_match(
                oslo_i18n.get_available_languages('neutron'))
        exc = api_common.convert_exception_to_http_exc(e, faults.FAULT_MAP,
                                                       language)
        if hasattr(exc, 'code') and 400 <= exc.code < 500:
            LOG.info('%(action)s failed (client error): %(exc)s',
                     {'action': state.request.method, 'exc': exc})
        else:
            LOG.exception('%s failed.', state.request.method)
        return exc



# Copyright (c) 2012 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api import attributes as attrs
from neutron_lib.api.definitions import network as net_def
from neutron_lib.api.definitions import port as port_def
from neutron_lib.api.definitions import subnet as subnet_def
from neutron_lib.api.definitions import subnetpool as subnetpool_def


# Defining a constant to avoid repeating string literal in several modules
SHARED = 'shared'

# Define constants for base resource name
CORE_RESOURCES = {net_def.RESOURCE_NAME: net_def.COLLECTION_NAME,
                  subnet_def.RESOURCE_NAME: subnet_def.COLLECTION_NAME,
                  subnetpool_def.RESOURCE_NAME: subnetpool_def.COLLECTION_NAME,
                  port_def.RESOURCE_NAME: port_def.COLLECTION_NAME}

RESOURCE_ATTRIBUTE_MAP = attrs.RESOURCES

# Identify the attribute used by a resource to reference another resource

RESOURCE_FOREIGN_KEYS = {
    net_def.COLLECTION_NAME: 'network_id'
}


def get_collection_info(collection):
    """Helper function to retrieve attribute info.

    :param collection: Collection or plural name of the resource
    """
    return RESOURCE_ATTRIBUTE_MAP.get(collection)
